/*global jQuery, mediaWiki, apiSandbox*/
/*jslint regexp: true, browser: true, continue: true, sloppy: true, white: true, forin: true, plusplus: true */
( function ( $, mw, apiSandbox, undefined ) {

	var mainRequest, genericRequest, generatorRequest, queryRequest, // UiBuilder objects
		// Caches
		paramInfo,
		// page elements
		$format, $action, $query, $queryRow, $help, $mainContainer, $genericContainer,
		$generatorContainer, $queryContainer, $generatorBox, $form, $submit, $requestUrl, $requestPost,
		$output, $postRow, $buttonsContainer, $examplesButton, $examplesContent, $pageScroll,
		UiBuilder = apiSandbox.UiBuilder;

	/** Local utility functions **/

	// get the first element in a list that is "scrollable"
	// depends on browser and skin (i.e. body or html)
	function getScrollableElement( /* selectors, ... */ ) {
		var i, argLen, el, $el, canScroll;
		for ( i = 0, argLen = arguments.length; i < argLen; i += 1 ) {
			el = arguments[i];
			$el = $( el );
			if ( $el.scrollTop() > 0 ) {
				return el;
			} else {
				$el.scrollTop( 1 );
				canScroll = $el.scrollTop() > 0;
				$el.scrollTop( 0 );
				if ( canScroll ) {
					return el;
				}
			}
		}
		return [];
	}

	function updateGenerator( callback ) {
		var generator = $( '#param-generator' ).val();
		if ( generator === '' ) {
			$generatorBox.hide();
		} else {
			$generatorBox.show();
			getParamInfo(
				{ querymodules: generator },
				function () {
					showLoading( $generatorContainer );
				},
				function () {
					generatorRequest = new UiBuilder( $generatorContainer, paramInfo.querymodules[generator], 'g' );
					if ( typeof callback === 'function' ) {
						callback();
					}
				},
				function () {
					showLoadError( $generatorContainer, 'apisb-request-error' );
				}
			);
		}
	}

	/**
	 * Displays a spinner and a "Loading..."
	 * @param $element {jQuery} Container for the loading message
	 */
	function showLoading( $element ) {
		$element.html(
			mw.html.element( 'img',
				{ src: mw.config.get( 'stylepath' ) + '/common/images/spinner.gif', alt: '' } ) +
			mw.html.escape( mw.msg( 'apisb-loading' ) )
		);
	}

	function showLoadError( $element, message ) {
		$element.html(
			mw.html.element( 'span', { 'class': 'error' }, mw.msg( message ) )
		);
	}

	/**
	 *
	 * @param what {object} Object with properties defining which information to retrieve, e.g. {modules:'query'}
	 * @param loadCallback {function} Callback called before retrieval of information from API starts
	 * @param completeCallback {function} Callback called when information is ready, either from API or cache
	 * @param errorCallback {function} Callback called when API call fails
	 */
	function getParamInfo( what, loadCallback, completeCallback, errorCallback ) {
		var needed, param, subParam;

		needed = {};
		// Remove the stuff we already have in cache from the API call parameters
		for ( param in what ) {
			if ( paramInfo[param] === undefined ) {
				needed[param] = what[param];
			} else if ( typeof needed[param] === 'object' ) {
				for ( subParam in param ) {
					if ( paramInfo[param][subParam] === undefined ) {
						needed[param][subParam] = what[param][subParam];
					}
				}
			} else {
				needed[param] = what[param];
			}
		}
		if ( $.isEmptyObject( needed ) ) {
			// Everything's in cache
			completeCallback();
		} else {
			loadCallback();
			needed.format = 'json';
			needed.action = 'paraminfo';
			$.getJSON(
				mw.util.wikiScript( 'api' ),
				needed,
				function ( data ) {
					var prop, i, info;

					if ( data.error || !data.paraminfo ) {
						errorCallback();
						return;
					}
					for ( prop in data.paraminfo ) {
						if ( paramInfo[prop] === undefined ) {
							paramInfo[prop] = data.paraminfo[prop];
						} else {
							for ( i = 0; i < data.paraminfo[prop].length; i++ ) {
								info = data.paraminfo[prop][i];
								if ( !paramInfo[prop].hasOwnProperty( info.name ) ) {
									paramInfo[prop][info.name] = info;
								}
							}
						}
					}
					completeCallback();
				}
			).error( errorCallback );
		}
	}

	/**
	 * Resets all form fields
	 */
	function resetUI() {
		$( '.api-sandbox-builder' ).each( function () {
			$( this ).data( 'builder' ).createInputs();
		} );
	}

	/**
	 * Sets the selected element(s) of a <select> input
	 * @param $el {jQuery} Element to modify
	 * @param value {string} Value(s) to select
	 */
	function setSelect( $el, value ) {
		var i, splitted;
		if ( $el.attr( 'multiple' ) ) {
			splitted = value.split( '|' );
			for ( i = 0; i < splitted.length; i++ ) {
				$el.find( 'option[value="' + mw.html.escape( splitted[i] ) + '"]' )
					.prop( 'selected', true );
			}
		} else {
			$el.find( 'option[value="' + mw.html.escape( value ) + '"]' )
				.prop( 'selected', true );
		}
	}

	/**
	 * @context {Element}
	 * @param e {jQuery.Event}
	 */
	function exampleClick( e ) {
		var link;
		e.preventDefault();

		resetUI();
		link = $( this ).data( 'exampleLink' ).replace( /^.*?\?/, '' );
		applyParams( link );
	}

	/**
	 * Sets form fields according to a URI query string
	 * @param link {string} Query string to apply
	 */
	function applyParams( link ) {
		var params, i, obj, pieces, key, value;
		params = link.split( '&' );
		obj = {};
		for ( i = 0; i < params.length; i++ ) {
			pieces = params[i].split( '=' );
			if ( pieces.length === 1 ) { // checkbox
				obj[pieces[0]] = null;
			} else {
				key = pieces[0];
				value = decodeURIComponent( pieces.slice( 1 ).join( '=' ) );
				obj[key] = value;
			}
		}
		applyObject( obj );

		/**
		 * Sets form fields according to an object properties
		 * @param obj {object} Object with properties representing an API request, e.g. {action:'edit',format:'json'}
		 */
		function applyObject( obj ) {
			var blacklist = [];

			// Set action
			if ( obj.action ) {
				setSelect( $action, obj.action );
				updateUI( function () {
					blacklist.push( 'action' );
					applyListParameter( obj, blacklist );
				}, true );
			} else {
				applyGeneratorParameter( obj, blacklist );
			}
		}

		function applyListParameter( obj, blacklist ) {
			var query;
			// Set query module, if any
			if ( obj.list ) {
				blacklist.push( 'list' );
				query = 'list=' + obj.list;
			} else if ( obj.prop ) {
				blacklist.push( 'prop' );
				query = 'prop=' + obj.prop;
			} else if ( obj.meta ) {
				blacklist.push( 'meta' );
				query = 'meta=' + obj.meta;
			}
			if ( query ) {
				setSelect( $query, query );
				updateUI( function () {
					applyGeneratorParameter( obj, blacklist );
				}, true );
			} else {
				applyGeneratorParameter( obj, blacklist );
			}
		}

		function applyGeneratorParameter( obj, blacklist ) {
			// Set generator, if any
			if ( obj.generator ) {
				setSelect( $( '#param-generator' ), obj.generator );
				updateGenerator( function () {
					applyRemainingFieldParameters( obj, blacklist );
				} );
			} else {
				applyRemainingFieldParameters( obj, blacklist );
			}
		}

		function applyRemainingFieldParameters( obj, blacklist ) {
			var key, value, $el, nodeName;
			// Set the remaining fields
			for ( key in obj ) {
				if ( obj.hasOwnProperty( key ) && blacklist.indexOf( key ) === -1 ) {
					value = obj[ key ];
					$el = $( '#param-' + key );
					if ( value === null ) { // checkbox
						$( '#param-' + key ).prop( 'checked', true );
					} else if ( $el[ 0 ] ) {
						nodeName = $el[ 0 ].nodeName.toLowerCase();
						switch ( nodeName ) {
							case 'select':
								setSelect( $el, value );
								break;
							case 'input':
								if ( $el.attr( 'type' ) === 'checkbox' ) {
									$( '#param-' + key ).prop( 'checked', true );
								} else {
									$el.val( value );
								}
								break;
							default:
								mw.log( 'Unrecognised node name "' + nodeName + '"' );
						}
					}
				}
			}
		}
	}

	function updateExamples( info ) {
		var i, $list, urlRegex, count, href, text, match, prefix, $prefix, linkText;

		if ( info.allexamples === undefined ) {
			return;
		}
		// on 1.18, convert everything into 1.19 format
		if ( info.allexamples.length > 0 && typeof info.allexamples[0] === 'string' ) {
			for ( i = 0; i < info.allexamples.length; i++ ) {
				info.allexamples[i] = { '*': info.allexamples[i] };
			}
		}
		$examplesContent.hide().html( '' );
		$list = $( '<ul>' );
		urlRegex = /api.php\?\S+/m;
		count = 0;
		for ( i = 0; i < info.allexamples.length; i++ ) {
			href = '';
			text = '';
			while ( i < info.allexamples.length && info.allexamples[i].description === undefined ) {
				match = urlRegex.exec( info.allexamples[i]['*'] );
				if ( match ) {
					href = match[0];
					break;
				} else {
					text += '\n' + info.allexamples[i]['*'];
				}
				i++;
			}
			if ( !href ) {
				href = info.allexamples[i]['*'];
			}
			if ( !text ) {
				text = info.allexamples[i].description !== undefined ? info.allexamples[i].description : href;
			}
			prefix = text.replace( /[^\n]*$/, '' );
			$prefix = prefix.length ? $( '<b>' ).text( prefix ) : [];
			linkText = text.replace( /^.*\n/, '' );
			$( '<li>' )
				.append( $prefix )
				.append(
					$( '<a>' )
						.attr( 'href', '#' )
						.data( 'exampleLink', href )
						.text( linkText )
						.click( exampleClick )
				).appendTo( $list );
			count++;
		}
		$examplesButton.button( 'option', 'text', mw.msg( count === 1 ? 'apisb-example' : 'apisb-examples' ) );
		$list.appendTo( $examplesContent );
		if ( count ) {
			$examplesButton.show();
		} else {
			$examplesButton.hide();
		}
	}

	function updateQueryInfo( action, query, callback ) {
		var data,
			isQuery = action === 'query';

		if ( action === '' || ( isQuery && query === '' ) ) {
			$submit.button( 'option', 'disabled', true );
			return;
		}
		query = query.replace( /^.*=/, '' );
		data = {};
		if ( isQuery ) {
			data.querymodules = query;
		} else {
			data.modules = action;
		}
		getParamInfo( data,
			function () {
				showLoading( $mainContainer );
				$submit.button( 'option', 'disabled', true );
				$examplesContent.hide();
			},
			function () {
				var info;
				if ( isQuery ) {
					info = paramInfo.querymodules[query];
				} else {
					info = paramInfo.modules[action];
				}
				mainRequest = new UiBuilder( $mainContainer, info, '' );
				mainRequest.setHelp( $help );
				$submit.button( 'option', 'disabled', false );
				updateExamples( info );
				if ( typeof callback === 'function' ) {
					callback();
				}
			},
			function () {
				$submit.button( 'option', 'disabled', false );
				showLoadError( $mainContainer, 'apisb-load-error' );
				$examplesContent.hide();
			}
		);
	}

	/**
	 * Updates UI after basic query parameters have been changed
	 */
	function updateUI( callback, callIfEmpty ) {
		var a = $action.val(),
			q = $query.val(),
			isQuery = a === 'query';
		if ( isQuery ) {
			$queryRow.show();
			if ( q !== '' ) {
				$queryContainer.show();
			} else {
				$queryContainer.hide();
			}
		} else {
			$queryRow.hide();
			$queryContainer.hide();
		}
		$mainContainer.text( '' );
		$help.text( '' );
		updateQueryInfo( a, q, callback );
		$generatorBox.hide();
		if ( q === '' && callIfEmpty && typeof callback === 'function' ) {
			callback();
		}
	}

	function runQuery( options ) {
		$.ajax( {
			url: options.url,
			data: options.params || {},
			dataType: 'text',
			type: options.type || 'GET',
			success: function ( origData, textStatus, jqXHR ) {
				var data = origData, match, time;

				if ( /html/.test( jqXHR.getResponseHeader( 'Content-Type' ) ) ) {
					match = data.match( /<pre[ >][\s\S]*<\/pre>/ );
				}

				if ( $.isArray( match ) ) {
					time = data.match( /<!-- Served .*?in (\d+(\.\d+)?) secs. -->/ );
					data = match[0];
					if ( $.isArray( time ) ) {
						data += '\n<br/>' + mw.html.escape( mw.msg( 'apisb-request-time', time[1] ) );
					}
				} else {
					// some actions don't honor user-specified format
					data = '<pre>' + mw.html.escape( data ) + '</pre>';
				}
				$output.html( data );
				if ( options.success ) {
					options.success( origData );
				}
			},
			error: function () {
				showLoadError( $output, 'apisb-request-error' );
			},
			// either success or error
			complete: function () {
				$pageScroll.animate( { scrollTop: $( '#api-sandbox-result' ).offset().top }, 400 );
			}
		} );
	}

	/** When the dom is ready... **/

	$( function () {
		$( '#api-sandbox-content' ).show();

		// init page elements
		$format = $( '#api-sandbox-format' );
		$action = $( '#api-sandbox-action' );
		$query = $( '#api-sandbox-query' );
		$queryRow = $( '#api-sandbox-query-row' );
		$help = $( '#api-sandbox-help' );
		$mainContainer = $( '#api-sandbox-main-inputs' );
		$generatorContainer = $( '#api-sandbox-generator-inputs' );
		$queryContainer = $( '#api-sandbox-query-inputs' );
		$generatorBox = $( '#api-sandbox-generator-parameters' );
		$requestUrl = $( '#api-sandbox-url' );
		$requestPost = $( '#api-sandbox-post' );
		$output = $( '#api-sandbox-output' );
		$postRow = $( '#api-sandbox-post-row' );
		$buttonsContainer = $( '#api-sandbox-buttons' );
		$examplesContent = $( '#api-sandbox-examples' );
		$pageScroll = $( getScrollableElement( 'body', 'html' ) );
		$form = $( '#api-sandbox-form' );
		$submit = $( '<button>' )
			.attr( 'type', 'submit' )
			.text( mw.msg( 'apisb-submit' ) )
			.appendTo( $buttonsContainer );
		$submit = $submit.clone( /*dataAndEvents=*/true, /*deep=*/true )
			.appendTo( '#api-sandbox-parameters' )
			.add( $submit )
			.button( { disabled: true } );

		$examplesButton = $( '<button>' )
			.attr( 'type', 'button' )
			.text( mw.msg( 'apisb-examples' ) )
			.click( function ( e ) {
				$examplesContent.slideToggle();
			} )
			.button()
			.hide()
			.appendTo( $buttonsContainer );

		$( '<button>' )
			.attr( 'type', 'button' )
			.text( mw.msg( 'apisb-clear' ) )
			.click( function ( e ) {
				resetUI();
			} )
			.button()
			.appendTo( $buttonsContainer );

		// init caches
		paramInfo = { modules: {}, querymodules: {} };

		// load stuff we need from the beginning
		getParamInfo(
			{
				mainmodule: 1,
				modules: 'query'
			},
			function () {
			},
			function () {
				paramInfo.mainmodule.parameters = paramInfo.mainmodule.parameters.slice( 2 ); // remove format and action
				paramInfo.modules.query.parameters = paramInfo.modules.query.parameters.slice( 3 );
				$genericContainer = $( '#api-sandbox-generic-inputs > div' );
				genericRequest = new UiBuilder( $genericContainer, paramInfo.mainmodule, '' );
				queryRequest = new UiBuilder( $queryContainer, paramInfo.modules.query, '' );
			},
			function () {
			}
		);

		$action.change( function () {
			updateUI();
		} );
		$query.change( function () {
			updateUI();
		} );

		$( '#param-generator' ).live( 'change', updateGenerator );

		function doHash() {
			var hash = window.location.hash.replace( /^#/, '' );
			applyParams( hash );
		}

		$( window ).on( 'popstate', function ( e ) {
			if ( genericRequest ) {
				genericRequest.createInputs();
			}
			if ( generatorRequest ) {
				generatorRequest.createInputs();
			}
			if ( queryRequest ) {
				queryRequest.createInputs();
			}
			doHash();
		} );

		$form.submit( function ( e ) {
			var url, params, mustBePosted, historyEntry, formatStr;

			// Prevent browser from submitting the form
			// and reloading the page to the action-url.
			// We're doing it with AJAX instead, below.
			e.preventDefault();

			if ( $submit.button( 'option', 'disabled' ) === true ) {
				return;
			}

			url = $.param( { action: $action.val() } );
			params = mainRequest.getRequestData();
			mustBePosted = mainRequest.info.mustbeposted === '';
			if ( $action.val() === 'query' ) {
				url += '&' + $query.val();
				params += queryRequest.getRequestData();
			}
			formatStr = $format.val();
			url += '&format=' + formatStr;

			params += genericRequest.getRequestData();
			if ( $( '#param-generator' ).length && $( '#param-generator' ).val() ) {
				params += generatorRequest.getRequestData();
			}

			historyEntry = '#' + url + params;

			if ( window.location.hash !== historyEntry && history && history.pushState ) {
				history.pushState( null, '', historyEntry );
			}
			url = mw.util.wikiScript( 'api' ) + '?' + url;

			showLoading( $output );
			if ( mustBePosted ) {
				$requestUrl.val( url );
				if ( params.length > 0 ) {
					params = params.substr( 1 ); // remove leading &
				}
				$requestPost.val( params );
				$postRow.show();
			} else {
				$requestUrl.val( url + params );
				$postRow.hide();
			}
			// There is no nonefm format
			if ( formatStr !== 'none' ) {
				url = url.replace( /(&format=[^&]+)/, '$1fm' );
			}
			runQuery( { url: url, params: params,
				type: mustBePosted ? 'POST' : 'GET' } );
		} );
		doHash();
	} );
}( jQuery, mediaWiki, mw.apiSandbox ) );
