<?php
/**
 * Extension ZeroRatedMobileAccess — Zero Rated Mobile Access
 *
 * @file
 * @ingroup Extensions
 * @author Patrick Reilly
 * @author Yuri Astrakhan
 * @copyright © 2011 Patrick Reilly, 2013 Yuri Astrakhan
 * @licence GNU General Public Licence 2.0 or later
 */

// Needs to be called within MediaWiki; not standalone
if ( !defined( 'MEDIAWIKI' ) ) {
	echo( "This is a MediaWiki extension and cannot run standalone.\n" );
	die( -1 );
}

// Extension credits that will show up on Special:Version
$wgExtensionCredits['other'][] = array(
	'path'  =>  __FILE__,
	'name'  =>  'ZeroRatedMobileAccess',
	'version'  =>  '1.1.0',
	'author' => array( 'Patrick Reilly', 'Yuri Astrakhan' ),
	'descriptionmsg'  =>  'zero-rated-mobile-access-desc',
	'url'  =>  'https://www.mediawiki.org/wiki/Extension:ZeroRatedMobileAccess',
);

$cwd = __DIR__ . DIRECTORY_SEPARATOR;

// FIXME: Should this namespace file be declared only on meta?
$wgExtensionMessagesFiles['ZeroRatedMobileAccessNamespaces'] = $cwd . 'ZeroRatedMobileAccess.namespaces.php';
$wgExtensionMessagesFiles['ZeroRatedMobileAccess'] = $cwd . 'ZeroRatedMobileAccess.i18n.php';

$cwd = $cwd . 'includes' . DIRECTORY_SEPARATOR;
$wgExtensionMessagesFiles['ZeroRatedMobileAccessConfig'] = $cwd . 'ZeroConfig.i18n.php';

// autoload extension classes

$autoloadClasses = array (
	'ApiZeroConfig' => 'ApiZeroConfig',
	'CarrierConfig' => 'CarrierConfig',
	'CarrierConfigStore' => 'CarrierConfigStore',
	'ConfigPageHooks' => 'ConfigPageHooks',
	'PageRenderingHooks' => 'PageRenderingHooks',
	'ZeroConfigContent' => 'ZeroConfigContent',
	'ZeroConfigContentHandler' => 'ZeroConfigContentHandler',
	'ZeroSpecialPage' => 'ZeroSpecialPage',
);

$ns = 'Extensions\ZeroRatedMobileAccess\\';
foreach ( $autoloadClasses as $className => $classFilename ) {
	$wgAutoloadClasses[$ns . $className] = $cwd . $classFilename . '.php';
}

$localBasePath = dirname( __FILE__ );
$remoteExtPath = 'ZeroRatedMobileAccess';

$wgResourceModules['mobile.zero.styles'] = array(
	'localBasePath' => $localBasePath,
	'remoteExtPath' => $remoteExtPath,
	'targets' => array( 'mobile' ),
	'styles' => array( 'modules/banner.css', 'modules/redux.less' ),
);

$wgResourceModules['mobile.zero'] = array(
	'dependencies' => array( 'mobile.stable', 'mobile.templates', 'mobile.zero.styles', 'mobile.overlays' ),
	'localBasePath' => $localBasePath,
	'remoteExtPath' => $remoteExtPath,
	'localTemplateBasePath' => $localBasePath . '/templates',
	'templates' => array( 'interstitial' ),
	'class' => 'MFResourceLoaderModule',
	'targets' => array( 'mobile' ),
	'scripts' => array( 'modules/ZeroOverlay.js', 'modules/banner.js', 'modules/interstitial.js' ),
	'messages' => array(
		'zero-rated-mobile-access-charge-auth',
		'zero-rated-mobile-access-file-auth',
		'zero-rated-mobile-access-accept',
		'zero-rated-mobile-access-go-back',
		'zero-rated-mobile-access-dont-ask',
	)
);

$wgResourceModules['zero.config'] = array(
	// banner styles are used for rendering example banners
	'localBasePath' => $localBasePath,
	'remoteExtPath' => $remoteExtPath,
	'styles'        => array( 'modules/zero.config.css', 'modules/banner.css' ),
	'position'      => 'top',
);

$wgResourceModules['zero.special.scripts'] = array(
	'localBasePath' => $localBasePath,
	'remoteExtPath' => $remoteExtPath,
	'targets' => array( 'mobile' ),
	'scripts' => array( 'modules/forms.js' ),
);

$wgResourceModules['zero.special.styles'] = array(
	'localBasePath' => $localBasePath,
	'remoteExtPath' => $remoteExtPath,
	'targets' => array( 'mobile' ),
	'styles' => array( 'modules/forms.css' ),
);

$wgEnableZeroRatedMobileAccessTesting = false;
$wgZeroRatedMobileAccessApiUri = false;
$wgZeroRatedMobileAccessDisableCache = false;
$wgZeroRatedMobileAccessEnableZeroConfigPages = false;
$wgZeroRatedMobileAccessEnableESI = false;

$wgHooks['BeforePageDisplayMobile'][] = $ns . 'PageRenderingHooks::onBeforePageDisplay';
$wgHooks['GetMobileUrl'][] = $ns . 'PageRenderingHooks::onGetMobileUrl';
$wgHooks['MinervaPreRender'][] = $ns . 'PageRenderingHooks::onMinervaPreRender';
$wgHooks['MobileFrontendBeforeDOM'][] = $ns . 'PageRenderingHooks::onMobileFrontendBeforeDOM';
$wgHooks['GetMobileNotice'][] = $ns . 'PageRenderingHooks::onGetMobileNotice';

define( 'NS_ZERO', 480 );
define( 'NS_ZERO_TALK', 481 );

// Only register NS_ZERO if running on the MediaWiki instance which
// houses the Zero configs (i.e., meta).
$wgHooks['CanonicalNamespaces'][] = function( array &$namespaces ) {
	global $wgZeroRatedMobileAccessEnableZeroConfigPages;
	if ( $wgZeroRatedMobileAccessEnableZeroConfigPages === true ) {
		$namespaces[NS_ZERO] = 'Zero';
		$namespaces[NS_ZERO_TALK] = 'Zero_talk';
	}
	return true;
};

// Zero configuration pages use a special, JSON-based content model.
$wgNamespaceContentModels[NS_ZERO] = 'JsonZeroConfig';
$wgContentHandlers['JsonZeroConfig'] = 'Extensions\ZeroRatedMobileAccess\ZeroConfigContentHandler';

// Registers hooks which are only required on the configuration wiki.
$wgExtensionFunctions[] = function() {
	global $wgHooks, $wgZeroRatedMobileAccessEnableZeroConfigPages, $wgAPIModules;

	$wgAPIModules['zeroconfig'] = 'Extensions\ZeroRatedMobileAccess\ApiZeroConfig';

	if ( $wgZeroRatedMobileAccessEnableZeroConfigPages === true ) {
		$hook = 'Extensions\ZeroRatedMobileAccess\ConfigPageHooks::';

		$wgHooks['MediaWikiPerformAction'][] = $hook . 'onMediaWikiPerformAction';
		$wgHooks['BeforePageDisplay'][] = $hook . 'onBeforePageDisplay';
		$wgHooks['EditFilterMergedContent'][] = $hook . 'onEditFilterMergedContent';
		$wgHooks['CodeEditorGetPageLanguage'][] = $hook . 'onCodeEditorGetPageLanguage';
		$wgHooks['PageContentSaveComplete'][] = $hook . 'onPageContentSaveComplete';
		$wgHooks['userCan'][] = $hook . 'onUserCan';

		return true;
	}

	return false;
};


$wgSpecialPages['ZeroRatedMobileAccess'] = $ns . 'ZeroSpecialPage';
$wgSpecialPageGroups['ZeroRatedMobileAccess'] = 'other';
