<?php


class ETLZeroData {
	public static function getWikiArticleData( $url ) {
		$rev = null;
		$jsonData = json_decode( file_get_contents( $url ), true );
		foreach ( $jsonData['query']['pages'] as $pages ) {
			if ( isset( $pages['revisions'][0]['*'] ) ) {
				$rev = $pages['revisions'][0]['*'];
			}
		}
		return $rev;
	}

	public static function asteriskedToArray( $articleData ) {
		$array = null;
		$currentIndex = null;
		$lines = explode( PHP_EOL, $articleData );
		foreach ( $lines as $line ) {
			if ( strpos( $line, '*' ) === 0 & strpos( $line, '**' ) !== 0 ) {
				$currentIndex = self::stripAsterisks( $line );
				$array[$currentIndex] = array();
			} elseif ( strpos( $line, '**' ) === 0 ) {
				$array[$currentIndex][] = self::stripAsterisks( $line );
			}
		}
		return $array;
	}

	public static function stripAsterisks( $string ) {
		return trim( str_replace( '*', '', $string ) );
	}

	public static function getRawHTTPBody( $url ) {
		return file_get_contents( $url );
	}

	public static function VCLToACLArray( $rawData ) {
		$aclRegex = '/acl( +)([a-zA-Z0-9_]+)( +){(.+?)}/s';
		preg_match_all( $aclRegex, $rawData, $matches );
		return $matches;

	}

	public static function getACLProperties( $aclArray, $rawText ) {
		$array = null;

		$aclRegexLeftHand = '/if \(client.ip ~ ';
		$aclRegexRightHandForXDfltLang = '\) {([^{]+?)set req\.http\.X-DfltLang = "([a-z]+)";/s';
		$aclRegexRightHandForXCarrier = '\) {([^}]+?)set req\.http\.X-Carrier = "([a-zA-Z ]+)";/s';
		$aclRegexRightHandForXCS = '\) {([^}]+?)set req\.http\.X-CS = "([a-zA-Z0-9\-*]+)";/s';
		$aclRegexRightHandForPermittedXSubdomain = '\) {([^{]+?)if \(req\.http\.X-Subdomain == "([a-zA-Z0-9\-]+)"/s';
		$aclRegexRightHandForWhitelistedLanguages = '\) {([^}]+?)if \(req\.http\.host ~ "\(\^\(([a-z\|]+)\)/s';

		foreach ( $aclArray[2] as $numericalIndex => $aclName ) {

			if ( $aclName === 'carrier_testing' ) {
				continue;
			}

			$array[$aclName]['acl_block'] = $aclArray[4][$numericalIndex];

			$regexForXDfltLang = $aclRegexLeftHand . $aclName . $aclRegexRightHandForXDfltLang;
			preg_match( $regexForXDfltLang, $rawText, $matchXDfltLang );
			$array[$aclName]['default_language'] = $matchXDfltLang ? $matchXDfltLang[2] : 'en';

			$regexForXCarrier = $aclRegexLeftHand . $aclName . $aclRegexRightHandForXCarrier;
			preg_match( $regexForXCarrier, $rawText, $matchXCarrier );
			$array[$aclName]['x_carrier'] = $matchXCarrier ? $matchXCarrier[2] : '!ERROR NO X-CARRIER!';

			$regexForXCS = $aclRegexLeftHand . $aclName . $aclRegexRightHandForXCS;
			preg_match( $regexForXCS, $rawText, $matchXCS );
			$array[$aclName]['x_cs'] = $matchXCS ? $matchXCS[2] : '!ERROR NO X-CS!';

			$regexForPermittedXSubodmain = $aclRegexLeftHand . $aclName . $aclRegexRightHandForPermittedXSubdomain;
			preg_match( $regexForPermittedXSubodmain, $rawText, $matchPermittedXSubdomain );
			if ( $matchPermittedXSubdomain ) {
				$array[$aclName]['permitted_x_subdomain'] = strtolower( $matchPermittedXSubdomain[2] );
			}

			$regexForWhitelistedLanguages = $aclRegexLeftHand . $aclName . $aclRegexRightHandForWhitelistedLanguages;
			preg_match( $regexForWhitelistedLanguages, $rawText, $matchWhitelistedLanguages );
			if ( $matchWhitelistedLanguages ) {
				$array[$aclName]['whitelisted_languages'] = $matchWhitelistedLanguages[2];
			}
		}

		return $array;
	}

	public static function foldInLanguageOrdering( &$carrierPreferencesArray, $languageOrderingArray ) {
		foreach ( $carrierPreferencesArray as $key => $subArray ) {
			$carrierPreferencesArray[$key][] = $languageOrderingArray[strtoupper( $key )];
		}
	}

	public static function foldInACLProperties( &$carrierPreferencesArray, $aclProperties ) {
		foreach ( $carrierPreferencesArray as $key => $subArray ) {
			foreach ( $aclProperties as $acl => $properties ) {
				if ( strcasecmp( $properties['x_carrier'], $key ) === 0 ) {
					$carrierPreferencesArray[$key][] = $aclProperties[$acl];
				}
			}
		}
	}

	public static function foldInTranslations( &$carrierPreferencesArray, $translations ) {
		foreach ( $carrierPreferencesArray as $key => $subArray ) {
			$hyphenatedCarrierName = str_replace( ' ', '-', strtolower( $key ) );
			$messageForCarrierBanner = 'zero-rated-mobile-access-banner-text-' . $hyphenatedCarrierName;
			$messageForCarrierName = 'zero-rated-mobile-access-banner-carrier-name-' . $hyphenatedCarrierName;

			$matchingBanners = array();
			$matchingNames = array();
			foreach ( $translations as $language => $messageValuePair ) {
				if ( $language !== 'qqq' ) {
					if ( array_key_exists( $messageForCarrierBanner, $messageValuePair ) ) {
						$matchingBanners[$language] = $messageValuePair[$messageForCarrierBanner];
					}
					if ( array_key_exists( $messageForCarrierName, $messageValuePair ) ) {
						$cn = $messageValuePair[$messageForCarrierName];
						if ( $language !== 'en' && ucwords ( $cn ) !== ucwords ( $translations['en'][$messageForCarrierName] ) ) {
							// nice hint at stackoverflow.com/questions/5259185
							$conversion = iconv( "ISO-8859-1", "UTF-8", $cn );

							if ( $cn !== $conversion ) {
								$matchingNames[$language] = $cn;
								// echo "adding $language $cn\n";
							} else {
								$matchingNames[$language] = ucwords( $cn );
								// echo "adding $language " . ucwords( $cn ) . "\n";
							}
						} elseif ( $language === 'en' ) {
							$matchingNames[$language] = ucwords( $cn );
							// echo "adding $language " . ucwords ( $cn ) . "\n";
						} else {
							// echo "not adding $language $cn\n";
						}
					}
				}
			}
			$carrierPreferencesArray[$key][] = $matchingBanners;
			$carrierPreferencesArray[$key][] = $matchingNames;
		}
	}

	public static function prepareArrayForJSONification( $carrierName, $array ) {
		$arrayOut = array();
		$arrayOut['xcs'] = $array[6]['x_cs']; // saving for a rainy day

		/* The following doesn't need to be set. If it's undefined, there's no special
		by-country exclusion of the 'Your carrier does not support Zero' warning verbiage
		$arrayOut['geographiesToSuppressZeroUnsupportedWarning'] = false;
		*/

		$hyphenatedAndLowercaseCarrierName = str_replace( ' ', '-', strtolower( $carrierName ) );

		// Keeping the following for a rainy day:
		// $arrayOut['messageId'] = 'zero-rated-mobile-access-banner-carrier-name-' . $hyphenatedAndLowercaseCarrierName;

		$regexForLanguageExtraction = '/([a-z]+) # ([0-9\.]+)%/';
		$languageArray = array();
		foreach ( $array[5] as $languageEntry ) {
			preg_match( $regexForLanguageExtraction, $languageEntry, $matches );
			if ( $matches[1] !== 'other' && $matches[1] !== 'portal' ) {
				if ( $hyphenatedAndLowercaseCarrierName === 'grameenphone-bangladesh' && $matches[1] === 'en' ) {
					continue;
				}
				$languageArray[$matches[1]] = floatval( $matches[2] );
			}
		}
		arsort( $languageArray );
		$arrayOut['showLangs'] = array_keys( $languageArray );

		if ( $hyphenatedAndLowercaseCarrierName === 'orange-congo' ) {
			$arrayOut['langNameOverrides'] = array( "kg" => "Kikongo" );
		}

		if ( $array[6]['whitelisted_languages'] ) {
			$arrayOut['whitelistedLangs'] = explode( '|', $array[6]['whitelisted_languages'] );
		} else {
			$arrayOut['whitelistedLangs'] = array();
		}

		$foregroundAndBackground = explode( ':', $array[3] );
		$arrayOut['foreground'] = $foregroundAndBackground[0];
		$arrayOut['background'] = $foregroundAndBackground[1];

		if ( $hyphenatedAndLowercaseCarrierName === 'saudi-telecom' ) {
			$arrayOut['fontSize'] = '0.9em';
		}

		$arrayOut['bannerWarning'] = strcasecmp( $array[4], 'BANNER_WARNING_ON' ) === 0 ? true : false;

		$arrayOut['showImages'] = strcasecmp( $array[2], 'IMAGES_ON' ) === 0 ? true : false;

		if ( count( $arrayOut['showLangs'] ) <= 1 ) {
			$arrayOut['showZeroPage'] = false;
		}

		$arrayOut['bannerUrl'] = $array[0];

		if ( $array[6]['permitted_x_subdomain'] ) {
			$arrayOut['sites'] = $array[6]['permitted_x_subdomain'];
		}

		$IPblocks = array();
		$deWhitespacedIPs = str_replace( array( "\t", "\n" ), '', $array[6]['acl_block'] );
		$deSemicolonedIPs = explode( ';', $deWhitespacedIPs );
		foreach ( $deSemicolonedIPs as $block ) {
			$deQuotedBlock = str_replace( '"', '', $block );
			if ( strlen( $deQuotedBlock ) ) {
				$ipBlock = explode( '/', $deQuotedBlock );
				if ( false !== filter_var( $ipBlock[0], FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE ) ) {
					$IPblocks[] = $deQuotedBlock;
				}
			}
		}
		$arrayOut['ips'] = $IPblocks;

		$arrayOut['banner'] = $array[7];
		$arrayOut['name'] = $array[8];

		return $arrayOut;
	}

	public static function tabifyArrayWithBlankDefaults( $array ) {
		$xcs = $array['xcs'];
		$enabled = "true";
		$sites = $array['sites'] ? $array['sites'] : "both";
		$names = array();
		foreach ( $array['name'] as $langCode => $name ) {
			$name = str_replace( "\t", ' ', $name );
			$names[] = "{$langCode}:{$name}";
			if ( strpos( $name, '~' ) ) {
				echo "\n\nSIRENS! TILDE FOUND IN \$name:$name\n\n";
			}
		}
		$namesCommaDelimited = implode( ',', $names );

		$banners = array();
		foreach ( $array['banner'] as $langCode => $banner ) {
			$banner = str_replace( "\t", ' ', $banner );
			$banners[] = "{$langCode}:{$banner}";
			if ( strpos( $banner, '~' ) ) {
				echo "\n\nSIRENS! TILDE FOUND IN \$banner:$banner\n\n";
			}
		}
		$bannersCommaDelimited = implode( ',', $banners );

		$bannerUrl = $array['bannerUrl'];

		$showLangs = implode( '~', $array['showLangs'] );
		$whitelistedLangsCommaDelimited = implode( '~', $array['whitelistedLangs'] );
		if ( $whitelistedLangsCommaDelimited === '' ) {
			$whitelistedLangsCommaDelimited = '(all)';
		}

		$langNameOverrides = array();
		$langNameOverridesTildeDelimited = '(none)';

		if ( array_key_exists( 'langNameOverrides', $array ) ) {
			foreach ( $array['langNameOverrides'] as $langCode => $translation ) {
				$langNameOverrides[] = "{$langCode}:{$translation}";
			}
			$langNameOverridesTildeDelimited = implode( '~', $langNameOverrides );
		}

		$background = $array['background'];
		$foreground = $array['foreground'];
		$fontSize = $array['fontSize'] ? $array['fontSize'] : '(default)';
		$bannerWarning = $array['bannerWarning'] ? 'true' : 'false';
		$showImages = $array['showImages'] ? "true" : "false";
		$showZeroPage = false === $array['showZeroPage'] ? 'false' : 'true';

		$ips = implode( '~', $array['ips'] );

		return "$xcs\t$enabled\t$sites\t$namesCommaDelimited\t$bannersCommaDelimited\t$bannerUrl\t" .
			"$showLangs\t$whitelistedLangsCommaDelimited\t$langNameOverridesTildeDelimited\t" .
			"$background\t$foreground\t$fontSize\t$bannerWarning\t$showImages\t$showZeroPage\t" .
			"$ips";
	}


	public static function getLoginInitializers( $username, $password, $endpoint ) {
		$nameValuePairs = array(
			'format' => 'json',
			'action' => 'login',
			'lgname' => $username,
			'lgpassword' => $password
		);

		$postData = http_build_query( $nameValuePairs );

		$httpPOSTOptions = array(
			'http' => array(
				'method' => 'POST',
				'header' => "Content-Type: application/x-www-form-urlencoded\r\n",
				'content' => $postData
			)
		);

		$ctx = stream_context_create( $httpPOSTOptions );

		$result = file_get_contents( $endpoint, false, $ctx );
		// echo "$result\n";
		$json = json_decode( $result, true );
		$token = $json['login']['token'];
		$sessionid = $json['login']['sessionid'];
		$prefix = $json['login']['cookieprefix'];

		return array( $token, $sessionid, $prefix );
	}

	public static function authenticate( $username, $password, $loginToken, $sessionid, $prefix, $endpoint ) {
		$nameValuePairs = array(
			'format' => 'json',
			'action' => 'login',
			'lgname' => $username,
			'lgpassword' => $password,
			'lgtoken' => $loginToken,
		);


		$postData = http_build_query( $nameValuePairs );

		$ch = curl_init( $endpoint );
		curl_setopt( $ch, CURLOPT_COOKIEJAR, 'cookiemonster.txt' );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_HTTPHEADER, array( "Cookie: {$prefix}_session={$sessionid}" ) );
		curl_setopt( $ch, CURLOPT_POST, true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $postData );

		$result = curl_exec( $ch );
		// echo "authentication result: $result\n";
		$json = json_decode( $result, true );
		$authenticatedSessiond = $json['login']['sessionid'];
		$lguserid = $json['login']['lguserid'];
		$lgusername = $json['login']['lgusername'];

		return array( $authenticatedSessiond, $lguserid, $lgusername );
	}


	public static function getEditToken( $sessionid, $lguserid, $lgusername, $prefix, $title, $endpoint ) {
		$nameValuePairs = array(
			'format' => 'json',
			'action' => 'query',
			'prop' => 'info|revisions',
			'intoken' => 'edit',
			'titles' => $title
		);

		$queryString = http_build_query( $nameValuePairs );

		$ch = curl_init( "{$endpoint}?{$queryString}" );
		curl_setopt( $ch, CURLOPT_COOKIEFILE, 'cookiemonster.txt' );
		curl_setopt( $ch, CURLOPT_COOKIEJAR, 'cookiemonster.txt' );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_HTTPGET, true );

		$result = curl_exec( $ch );
		$json = json_decode( $result, true );

		foreach ( $json['query']['pages'] as $page ) {
			if ( array_key_exists( 'edittoken', $page ) ) {
				return $page['edittoken'];
			}
		}
		exit; // uh oh!
	}

	public static function postConfiguration( $sessionid, $lguserid, $lgusername, $prefix, $editToken, $title, $content, $endpoint ) {

		$nameValuePairs = array(
			'format' => 'json',
			'action' => 'edit',
			'title' => $title,
			'text' => $content,
			'token' => $editToken
		);

		$postData = http_build_query( $nameValuePairs );

		$ch = curl_init( $endpoint );
		curl_setopt( $ch, CURLOPT_COOKIEFILE, 'cookiemonster.txt' );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_POST, true );
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $postData );

		$result = curl_exec( $ch );
		// echo "raw json result: $result\n";
		return json_decode( $result, true );
	}
}

if ( $argc < 2 ) {
	echo "Sorry, no can do.\n";
	echo "USAGE:\nphp " . __FILE__ . " tsv\n" .
		"php " . __FILE__ . " push <apibeta|metaprod> <username> <password>\n" .
		"php " . __FILE__ . " both <apibeta|metaprod> <username> <password>\n";
	exit;
}

$tsvPushBoth = null;

if ( $argv[1] !== 'tsv' && $argv[1] !== 'push' && $argv[1] !== 'both' ) {
	echo "ERROR: Behavior must be tsv, push, or both.\n";
	exit;
} else {
	$tsvPushBoth = $argv[1];
}

$apiUrl = null;
$destinationUrlPrefix = null;
$username = null;
$password = null;

if ( $tsvPushBoth !== 'tsv' ) {
	switch ( $argv[2] ) {
		case "apibeta":
			$apiUrl = 'http://api.beta.wmflabs.org/w/api.php';
			$destinationUrlPrefix = 'http://api.beta.wmflabs.org/wiki/Zero:';
			break;
		case "metaprod":
			$apiUrl = 'https://meta.wikimedia.org/w/api.php';
			$destinationUrlPrefix = 'https://meta.wikimedia.org/wiki/Zero:';
			break;
		default:
			echo "ERROR: Target location must be apibeta or metaprod.\n";
			exit;
	}

	if ( $argc !== 5 ) {
		echo "ERROR: You must specify a username and a password as your final arguments.\n";
		exit;
	} else {
		$username = $argv[3];
		$password = $argv[4];
		/*
		echo "\$username: $username \$password: $password\n";
		exit;
		*/
	}
}

$carrierPrefs = ETLZeroData::getWikiArticleData( "https://en.wikipedia.org/w/api.php?action=query&prop=revisions&&rvlimit=1&rvprop=content&format=json&titles=MediaWiki:Zero-rated-mobile-access-carrier-options" );
$languageOrdering = ETLZeroData::getWikiArticleData( "https://en.wikipedia.org/w/api.php?action=query&prop=revisions&&rvlimit=1&rvprop=content&format=json&titles=MediaWiki:Zero-rated-mobile-access-language-options-for-carrier" );
$carrierPreferencesArray = ETLZeroData::asteriskedToArray( $carrierPrefs );
$languageOrderingArray = ETLZeroData::asteriskedToArray( $languageOrdering );
#$vclSettings = ETLZeroData::getRawHTTPBody( "https://gerrit.wikimedia.org/r/gitweb?p=operations/puppet.git;a=blob_plain;f=templates/varnish/mobile-frontend.inc.vcl.erb;hb=HEAD" );
$vclSettings = ETLZeroData::getRawHTTPBody( "https://gerrit.wikimedia.org/r/gitweb?p=operations/puppet.git;a=blob_plain;f=templates/varnish/mobile-frontend.inc.vcl.erb;hb=887beb89d20165c3c64eca1ad483efbe86f7833f" );
$acls = ETLZeroData::VCLToACLArray( $vclSettings );
$aclProperties = ETLZeroData::getACLProperties( $acls, $vclSettings );
ETLZeroData::foldInLanguageOrdering( $carrierPreferencesArray, $languageOrderingArray );
ETLZeroData::foldInACLProperties( $carrierPreferencesArray, $aclProperties );
include_once( 'ZeroConfig.i18n.php' ); // defines $messages
ETLZeroData::foldInTranslations( $carrierPreferencesArray, $messages );

//print_r( $carrierPreferencesArray );
//exit;

$phpV54orHigher = version_compare( PHP_VERSION, '5.4.0' ) >= 0;

if ( $tsvPushBoth === 'tsv' || $tsvPushBoth === 'both' ) {
	echo "xcs\tenabled\tsites\tname\tbanner\tbannerUrl\t" .
		"showLangs\twhitelistedLangs\tlangNameOverrides\t" .
		"background\tforeground\tfontSize\tbannerWarning\tshowImages\tshowZeroPage\t" .
		"ips\n";
}


foreach ( $carrierPreferencesArray as $carrierName => $carrierPreferences ) {
	$xcs = $carrierPreferences[6]['x_cs'];
	$JSONifiedEntry = ETLZeroData::prepareArrayForJSONification( $carrierName, $carrierPreferences );

	if ( $tsvPushBoth === 'tsv' || $tsvPushBoth === 'both' ) {
		$tabifiedEntry = ETLZeroData::tabifyArrayWithBlankDefaults( $JSONifiedEntry );
		echo "$tabifiedEntry\n";
	}


	if ( $tsvPushBoth === 'push' || $tsvPushBoth === 'both' ) {
		$JSONifiedEntryWithoutXCS = $JSONifiedEntry;
		unset( $JSONifiedEntryWithoutXCS['xcs'] );

		$jsonData = null;
		// print_r( $JSONifiedEntryWithoutXCS );
		if ( $phpV54orHigher ) {
			$jsonData = str_replace( '\/', '/', json_encode( $JSONifiedEntryWithoutXCS, JSON_PRETTY_PRINT ) ) . "\n";
		} else {
			$jsonData = str_replace( '\/', '/', json_encode( $JSONifiedEntryWithoutXCS ) ) . "\n";
		}

		// echo "Last JSON Error: " . json_last_error() . "\n";

		$destinationUrl = "{$destinationUrlPrefix}{$xcs}";
		// echo "\$destinationUrl $destinationUrl\n";
		// echo "\$jsonData $jsonData\n";

		list ( $token, $sessionid, $prefix ) = ETLZeroData::getLoginInitializers( $username, $password, $apiUrl );
		// echo "\$token: $token \$sessionid $sessionid \$prefix $prefix\n";
		list ( $authenticatedSessionid, $lguserid, $lgusername ) = ETLZeroData::authenticate( $username, $password, $token, $sessionid, $prefix, $apiUrl );
		$editToken = ETLZeroData::getEditToken( $authenticatedSessionid, $lguserid, $lgusername, $prefix, 'Zero:' . $xcs, $apiUrl );

		// echo "\$editToken: $editToken\n";
		$ret = ETLZeroData::postConfiguration( $authenticatedSessionid, $lguserid, $lgusername, $prefix, $editToken, 'Zero:' . $xcs, $jsonData, $apiUrl );
		// print_r( $ret );
	}
}
