#! /bin/sh

export PATH=/bin:$PATH

echo "AUTOMATED TESTS STARTED"
phantomjs zero_automated_tests.js $1 $2 > fullreport.txt 2>&1
echo "AUTOMATED TESTS FINISHED"


echo "UNSAFE URLS BECAUSE OF SCHEME" > badlinks-scheme-base.txt
echo "=============================" >> badlinks-scheme-base.txt
echo "" >> badlinks-scheme-base.txt
grep 'UNSAFE URL UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' fullreport.txt | grep ' NORMAL ' >> badlinks-scheme-base.txt

echo "UNSAFE URLS BECAUSE OF SCHEME" > badlinks-scheme-cachebusted-n-random.txt
echo "=============================" >> badlinks-scheme-cachebusted-n-random.txt
echo "" >> badlinks-scheme-cachebusted-n-random.txt
grep 'UNSAFE URL UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' fullreport.txt | grep ' CACHEBUSTED OR RANDOM REDIRECTED ' >> badlinks-scheme-cachebusted-n-random.txt


echo "UNSAFE URLS BECAUSE OF DOMAIN" > badlinks-domain-base.txt
echo "=============================" >> badlinks-domain-base.txt
echo "" >> badlinks-domain-base.txt
grep 'UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' fullreport.txt | grep ' NORMAL ' >> badlinks-domain-base.txt

echo "UNSAFE URLS BECAUSE OF DOMAIN" > badlinks-domain-cachebusted-n-random.txt
echo "=============================" >> badlinks-domain-cachebusted-n-random.txt
echo "" >> badlinks-domain-cachebusted-n-random.txt
grep 'UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' fullreport.txt | grep ' CACHEBUSTED OR RANDOM REDIRECTED ' >> badlinks-domain-cachebusted-n-random.txt


echo "UNSAFE SUBRESOURCES BECAUSE OF SCHEME" > badsubresources-scheme-base.txt
echo "=====================================" >> badsubresources-scheme-base.txt
echo "" >> badsubresources-scheme-base.txt
grep 'UNSAFE SUBRESOURCE UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' fullreport.txt | grep ' NORMAL ' >> badsubresources-scheme-base.txt

echo "UNSAFE SUBRESOURCES BECAUSE OF SCHEME" > badsubresources-scheme-cachebusted-n-random.txt
echo "=====================================" >> badsubresources-scheme-cachebusted-n-random.txt
echo "" >> badsubresources-scheme-cachebusted-n-random.txt
grep 'UNSAFE SUBRESOURCE UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' fullreport.txt | grep ' CACHEBUSTED OR RANDOM REDIRECTED ' >> badsubresources-scheme-cachebusted-n-random.txt


echo "UNSAFE SUBRESOURCES BECAUSE OF DOMAIN" > badsubresources-domain-base.txt
echo "=====================================" >> badsubresources-domain-base.txt
echo "" >> badsubresources-domain-base.txt
grep 'UNSAFE SUBRESOURCE SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' fullreport.txt | grep ' NORMAL ' | grep -v ' (enabled=false) ' >> badsubresources-domain-base.txt

echo "UNSAFE SUBRESOURCES BECAUSE OF DOMAIN" > badsubresources-domain-cachebusted-n-random.txt
echo "=====================================" >> badsubresources-domain-cachebusted-n-random.txt
echo "" >> badsubresources-domain-cachebusted-n-random.txt
grep 'UNSAFE SUBRESOURCE SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' fullreport.txt | grep ' CACHEBUSTED OR RANDOM REDIRECTED ' | grep -v ' (enabled=false) ' >> badsubresources-domain-cachebusted-n-random.txt


echo "BAD ZERO RESOURCELOAD" > badzeroresourceload.txt
echo "=====================" >> badzeroresourceload.txt
echo "" >> badzeroresourceload.txt
grep 'BAD ZERO RESOURCELOAD WHEN ' fullreport.txt >> badzeroresourceload.txt

echo "BAD SPIDERED" > badspidered.txt
echo "============" >> badspidered.txt
echo "" >> badspidered.txt
grep ' FROM SPIDERED PAGE ' fullreport.txt | grep 'UNSAFE URL ' >> badspidered.txt


mv report.zip report.bak.zip
mv banners.zip banners.bak.zip
mv diffs_rl_tidy_wml.zip diffs_rl_tidy_wml.bak.zip

unzip report.bak.zip -d old
mkdir seds

sed 's/ on 20.*//g' old/badlinks-scheme-base.txt | sort | uniq > seds/badlinks-scheme-base-old.txt
sed 's/ on 20.*//g' badlinks-scheme-base.txt | sort | uniq > seds/badlinks-scheme-base-new.txt
diff seds/badlinks-scheme-base-old.txt seds/badlinks-scheme-base-new.txt > diff-badlinks-scheme-base.txt

sed 's/ on 20.*//g' old/badsubresources-scheme-base.txt | sort | uniq > seds/badsubresources-scheme-base-old.txt
sed 's/ on 20.*//g' badsubresources-scheme-base.txt | sort | uniq > seds/badsubresources-scheme-base-new.txt
diff seds/badsubresources-scheme-base-old.txt seds/badsubresources-scheme-base-new.txt > diff-badsubresources-scheme-base.txt


sed 's/ on 20.*//g' old/badlinks-domain-base.txt | sort | uniq > seds/badlinks-domain-base-old.txt
sed 's/ on 20.*//g' badlinks-domain-base.txt | sort | uniq > seds/badlinks-domain-base-new.txt
diff seds/badlinks-domain-base-old.txt seds/badlinks-domain-base-new.txt > diff-badlinks-domain-base.txt

sed 's/ on 20.*//g' old/badsubresources-domain-base.txt | sort | uniq > seds/badsubresources-domain-base-old.txt
sed 's/ on 20.*//g' badsubresources-domain-base.txt | sort | uniq > seds/badsubresources-domain-base-new.txt
diff seds/badsubresources-domain-base-old.txt seds/badsubresources-domain-base-new.txt > diff-badsubresources-domain-base.txt


DIFFCOUNT=`cat diff-badlinks-scheme-base.txt diff-badsubresources-scheme-base.txt diff-badlinks-domain-base.txt diff-badsubresources-domain-base.txt | wc -l`
RLLINES=`cat badzeroresourceload.txt | wc -l`
RLLINECOUNT=`expr $RLLINES - 3`

rm -rf old/
rm -rf seds/



LATEST=`cat latestreport`


echo "TIDY GRIPES" > tidy_gripes.txt
echo "===========" >> tidy_gripes.txt
echo "" >> tidy_gripes.txt

find ./$LATEST/ -name '*.content.html' -exec sh -c 'printf "\n\n\n{}\n" >> tidy_gripes.txt; tidy -utf8 -errors {} 2>> tidy_gripes.txt;' \;

TIDYCOUNT=`cat tidy_gripes.txt | wc -l`


grep 'WMLRECORD' fullreport.txt > wmlentries.txt
awk < wmlentries.txt '{ system("curl -H \"X-CS:"$3"\" -H \"Accept: text/vnd.wap.wml\" -H \"User-Agent: Nokia3330/1.0 (03.05)\" "$2" ") }' > wmlout.txt
grep '<html' wmlout.txt > wmlbad.txt
grep '<wml ' wmlout.txt > wmlgood.txt
WMLTOTAL=`cat wmlentries.txt | wc -l`
WMLGOOD=`cat wmlgood.txt | wc -l`
WMLBAD=`cat wmlbad.txt | wc -l`
WMLSTATUS="WML NOT FOUND!!!"
if [ "$WMLTOTAL" -gt "0" ]
then
    if [ "$WMLTOTAL" -eq "$WMLGOOD" -a "$WMLBAD" -eq "0" ]
    then
        WMLSTATUS="WML OK $WMLTOTAL Entries $WMLGOOD Good $WMLBAD Bad"
    else
        WMLSTATUS="WML FAILED $WMLTOTAL Entries $WMLGOOD Good $WMLBAD Bad"
    fi
fi
echo $WMLSTATUS

echo "ERRORS (INSPECT!)" > errors.txt
echo "=================" >> errors.txt
echo "" >> errors.txt
grep -v 'OKAY URL ' fullreport.txt | grep -v 'OKAY SUBRESOURCE ' | grep -v ' found hyperlink ' | grep -v ' loaded subresource ' | grep -v 'QUEUED URL ' | grep -v 'DOWNLOADING URL ' | grep -v 'SAVED FILE ' | grep -v 'Starting\.\.\.' | grep -v 'CREATED DIRECTORY ' | grep -v 'CONFIGURATION IDENTIFIED ' | grep -v 'LOADED MAIN CONFIGURATION PAGE AT ' | grep -v 'DOWNLOADED CONFIGURATION ' | grep -v 'COMMENT FOR ' | grep -v '\.\.\.Exiting\.\.\.' | grep -v 'Loaded homepage from ' | grep -v 'Top level URL ' | grep -v 'UNSAFE URL ' | grep -v 'UNSAFE SUBRESOURCE ' | grep -v 'BAD ZERO RESOURCELOAD WHEN ' | grep -v 'WMLRECORD'  | grep -v 'GOANYWHERE LINK VISITED' >> errors.txt


ERRORREPORTS="badlinks-scheme-base.txt badlinks-domain-base.txt badsubresources-scheme-base.txt badsubresources-domain-base.txt badlinks-scheme-cachebusted-n-random.txt badlinks-domain-cachebusted-n-random.txt badsubresources-scheme-cachebusted-n-random.txt badsubresources-domain-cachebusted-n-random.txt errors.txt badzeroresourceload.txt badspidered.txt wmlbad.txt"
ERRORLINES=`cat $ERRORREPORTS | wc -l`
ERRORLINECOUNT=`expr $ERRORLINES - 33`


zip report.zip $ERRORREPORTS fullreport.txt diff-badlinks-scheme-base.txt diff-badsubresources-scheme-base.txt diff-badlinks-domain-base.txt diff-badsubresources-domain-base.txt $LATEST/000.html $LATEST/*-topclip.png tidy_gripes.txt wmlentries.txt wmlout.txt wmlgood.txt latestreport
zip banners.zip $LATEST/000.html $LATEST/*-topclip.png
zip diffs_rl_tidy_wml.zip diff-badlinks-scheme-base.txt diff-badsubresources-scheme-base.txt diff-badlinks-domain-base.txt diff-badsubresources-domain-base.txt badlinks-scheme-base.txt badsubresources-scheme-base.txt errors.txt badlinks-domain-base.txt badsubresources-domain-base.txt badzeroresourceload.txt tidy_gripes.txt wmlentries.txt wmlout.txt wmlgood.txt wmlbad.txt latestreport

mv $ERRORREPORTS fullreport.txt diff-badlinks-scheme-base.txt diff-badsubresources-scheme-base.txt diff-badlinks-domain-base.txt diff-badsubresources-domain-base.txt tidy_gripes.txt wmlentries.txt wmlout.txt wmlbad.txt wmlgood.txt $LATEST/
cp latestreport $LATEST/


zip all.zip $LATEST/*
mv all.zip /var/www/banners/
cp $LATEST/000.html /var/www/banners/index.html
cp $LATEST/*-topclip.png /var/www/banners/

A=`date --rfc-3339=seconds`

echo "See attached for diffs on bad links and subresources ($DIFFCOUNT lines to inspect), plus superfluous ResourceLoader inclusion of Wikipedia Zero scripts and styles ($RLLINECOUNT lines to inspect), plus Tidy complaints ($TIDYCOUNT lines to inspect), plus WML outcomes ($WMLSTATUS) from the latest Wikipedia Zero automation test run. The diff output: less than symbols refer to the old report, greater than symbols the new report." | mailx -s "Wikipedia Zero Automated Test Bot Diff, RL, Tidy, WML for $A (Diff $DIFFCOUNT, RL $RLLINECOUNT, Tidy $TIDYCOUNT, $WMLSTATUS)" -a diffs_rl_tidy_wml.zip -c abaso@wikimedia.org yastrakhan@wikimedia.org

sleep 62

echo "See attached for live banners from the latest Wikipedia Zero automation test run." | mailx -s "Wikipedia Zero Automated Test Bot Banner Results for $A" -a banners.zip -c abaso@wikimedia.org dfoy@wikimedia.org

sleep 126

echo "See attached for Wikipedia Zero automation test outcomes ($ERRORLINECOUNT issues, WML status of $WMLSTATUS)." | mailx -s "Wikipedia Zero Automated Test Bot Results for $A ($ERRORLINECOUNT issues, $WMLSTATUS)" -a report.zip abaso@wikimedia.org

echo "DONE"
