/*

Based on https://github.com/ariya/phantomjs/blob/master/examples/render_multi_url.js

See the README for details.

The code would benefit from concreate cases abstracted to general functions. But it works.

Here's one feature that would be really nice to have.

 * X-Forwarded-For. Note that on zero-bdd.wmflabs.org a 10dot address is identified by Varnish.

Here are some other nice-to-haves for rainy days.

 * Check VCL behavior
 * IP-to-X-CS mapping
 * Opera
 * Get link count on Google.
 * Get operator's webpage. Set Accept: header if necessary.
 * Verify the banner matches what's expected, within reason.
   It may be possible to ape the {{SITENAME}}s, and then replace $1 and other stuff like that
 * Obtain message listings from translatewiki, or maybe easier, from PHP file in the Git repo.
 * Maybe screenshot Wikipedia Zero articles on wikimediafoundation.org, mediawiki.org, and
   en.wikipedia.org, and so on.
 * Exercise more UAs.
 * wikipedia.com, www.wikipedia.com, wikipedia.net, www.wikipedia.net, wikipedia.org, www.wikipedia.org behavior when partner network-sourced. Not currently necessary as the traffic is treated no differently.
 * Flag the extended attributes of the small list of <a>s that don't have an HREF.
 * Diff on "bad" where the starting page IS itself a cachebuster or Special:Random. This may be unnecessarily noisy, though.
 * Command line UA parameter customization
 *  * Resources loaded report on 3 keypresses into Search field
 * Click on menu and follow each safe link.
 * Consider using getAttribute('href') to obtain non-relative link status

 */

try {

    phantom.onError = function(msg, trace) {
        var msgStack = ['PHANTOM ERROR: ' + msg];
        if (trace && trace.length) {
            msgStack.push('TRACE:');
            trace.forEach(function(t) {
                msgStack.push(' -> ' + (t.file || t.sourceURL) + ': ' + t.line + (t.function ? ' (in function ' + t.function + ')' : ''));
            });
        }
        console.log(msgStack.join('\n'));
        phantom.exit(1);
    };

    console.log('Starting...' + ' on ' + (new Date()).toISOString());

    var ZRMA = {};

    ZRMA.mkdir = function() {
        var fs = require('fs');
        var now = new Date();
        var isoDateString = now.toISOString().replace(/:/g, '-');
        fs.makeDirectory(isoDateString);
        console.log('CREATED DIRECTORY ' + isoDateString);
        fs.write('latestreport', isoDateString, 'w');
        fs.write(
            isoDateString + '/000.html',
            '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en"><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /><style type="text/css">table, th, td { border: 1px solid black; }</style><title>Wikipedia Zero Automation Test Results for ' +
                isoDateString + '</title></head><body><h1>Wikipedia Zero Automation Test Results for ' +
                isoDateString + '</h1><p><a href="all.zip">Download Full Report</a></p><div><table>' +
                '<tr><th>Comment</th>' +
                '<th>X-CS</th>' +
                '<th>URL</th>' +
                '<th>Supported xdots</th>' +
                '<th>Whitelisted Langs</th>' +
                '<th>Config Enabled?</th>' +
                '<th>Lang Supported?</th>' +
                '<th>Real Sample</th></tr>' ,

            'w'
        );
        return isoDateString;
    };

    ZRMA.getSpeed = function () {

        var pargs = require('system').args;

        return pargs.length > 1 && pargs[1] === 'faster' ? 'faster' : 'normal';
    };

    ZRMA.getXcs = function() {
        var zConf;

        var pargs = require('system').args;
        if (pargs.length === 3) {
            var mccMnc = pargs[2].replace("-", "\\-");
            zConf = new RegExp("https:\\/\\/meta.wikimedia.org\\/wiki\\/Zero:(" + mccMnc + "+)");
        } else {
            zConf = /^https:\/\/meta.wikimedia.org\/wiki\/Zero:(\d+\-[0-9*]+)/;
            // zConf = /^https:\/\/meta.wikimedia.org\/wiki\/Zero:(470\-01)/;
            // zConf = /^https:\/\/meta.wikimedia.org\/wiki\/Zero:(2\d\d\-\d\d)/;
            // zConf = /^https:\/\/meta.wikimedia.org\/wiki\/Zero:(250\-99)/;
        }
        return zConf;
    };


    ZRMA.DoStuffWithUrls = function(urls, callbackPerUrl, callbackFinal) {
        // urlIndex can probably go away; it isn't used and it would exceed bounds because of unshift method!
        var fillBucket, addToBucket, getBucket, emptyBucket, next, page, retrieve, urlIndex, webpage, bucket = [];
        urlIndex = 0;
        webpage = require("webpage");
        page = null;

        fillBucket = function(b) {
            bucket = b;
        }

        addToBucket = function (c) {
            bucket.push(c);
        }

        getBucket = function() {
            return bucket;
        }

        emptyBucket = function () {
            bucket = [];
        }

        next = function(status, url) {
            callbackPerUrl(status, url, page, fillBucket, addToBucket);
            page.close();
            return retrieve();
        };
        retrieve = function() {
            var url;
            if (urls.length > 0) {
                url = urls.shift();
                urlIndex++;
                page = webpage.create();
                page.viewportSize = {
                    width: 320,
                    height: 480
                };

                page.settings.userAgent = url['ua'];
                page.customHeaders = url['headers'];

                page.onResourceRequested = function (req) {
                    var subresource = /^data:image\//.exec(req['url']) !== null ? 'data:image' : req['url'];
                    if (url['url'] !== subresource) {
                        console.log('Page ' + url['url'] + ' actually at ' + page.url + ' loaded subresource ' + decodeURI(subresource) + ' on ' + (new Date()).toISOString());
                    }

                    var xcsMissing = url['headers']['X-CS'] === undefined;
                    var xcsVerbiage = xcsMissing ? 'X-CS missing, ' : 'X-CS' + ' was ' + url['headers']['X-CS'] + ', ';
                    var configWasDisabled = url['enabled'] === false
                    var configDisabledVerbiage = configWasDisabled ? 'the configuration was disabled, ' : 'the configuration was indeterminate, ';
                    var languageOutOfBounds = url['whitelistedLangs'] !== undefined && url['whitelistedLangs'].length > 0 && url['whitelistedLangs'].indexOf(url['lang']) === -1 && url['subdomain'] !== 'zero';
                    var languageVerbiage = languageOutOfBounds ? ' and the language was not in the whitelist ' : ' and the language whitelist was indeterminate';

                    if ((xcsMissing ||
                         configWasDisabled ||
                         languageOutOfBounds
                        ) &&
                        url['url'] !== 'http://zero.wikipedia.org/'
                    ) {
                        rlExtractor = /mobile\.zero\.scripts|mobile\.zero\.styles|banner\.css|banner\.js|zero\.config\.css/;
                        // let's check for both RL'd and non-RL'd includes
                        if (rlExtractor.test(decodeURI(subresource)) || rlExtractor.test(subresource)) {
                            console.log('BAD ZERO RESOURCELOAD WHEN ' + xcsVerbiage + configDisabledVerbiage + languageVerbiage + ' : From ' + url['url'] + ' actually at ' + page.url + ' required subresource ' + decodeURI(subresource) + ' after ' + rlExtractor.source + ' was tested on ' + (new Date()).toISOString());
                        }
                    }

                    // TODO: once interstitial approach changes, revisit, remembering this section if for subresources
                    if (url['goAnywhere'] === true) {
                        hostExtractor = /^(?:http(?:s?):\/\/(.+?)\/.*|(data:image))$/;
                        domain = hostExtractor.exec(subresource);
                        console.log('OKAY SUBRESOURCE MATCHED (domain=' + domain[1] + ') WITH X-CS ' + url['headers']['X-CS'] + ' (enabled=' + url['enabled'] + ') : From ' + url['url'] + ' actually at ' + page.url + ' required subresource ' + decodeURI(subresource) + ' because of acceptbilling=yes redirect on ' + (new Date()).toISOString());
                    } else {
                        hostExtractor = /^(?:http:\/\/(.+?)\/.*|(data:image))$/;
                        domain = hostExtractor.exec(subresource);

                        if (url['safeDomains']) {
                            var cacheBustedOrRandomRx = /cachebuster=\d+|Special:Random/;
                            var cacheBustedOrRandomRxIndicator = cacheBustedOrRandomRx.test(url['url']) ?
                                'CACHEBUSTED OR RANDOM REDIRECTED' :
                                'NORMAL';
                            if (domain === null) {
                                console.log('UNSAFE SUBRESOURCE UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' + url['headers']['X-CS'] + ' (enabled=' + url['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + url['url'] + ' actually at ' + page.url + ' required subresource ' + decodeURI(subresource) + ' using regex ' + hostExtractor.source + ' on ' + (new Date()).toISOString())
                            } else {
                                domainMatch = url['safeDomains'].exec(domain[1]);
                                if (domainMatch === null && domain[1] !== 'data:image') {
                                    console.log('UNSAFE SUBRESOURCE SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' + url['headers']['X-CS'] + ' (enabled=' + url['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + url['url'] + ' actually at ' + page.url + ' required subresource ' + decodeURI(subresource) + ' using regex ' + url['safeDomains'].source + ' after ' + hostExtractor.source + ' identified domain ' + domain[1] + ' on ' + (new Date()).toISOString());
                                } else {
                                    console.log('OKAY SUBRESOURCE MATCHED (domain=' + domain[1] + ') WITH X-CS ' + url['headers']['X-CS'] + ' (enabled=' + url['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + url['url'] + ' actually at ' + page.url + ' required subresource ' + decodeURI(subresource) + ' using regexes ' + url['safeDomains'].source + ' and ' + hostExtractor.source + ' on ' + (new Date()).toISOString());
                                }
                            }
                        }

                    }
                };

                page.onResourceReceived = function(res) {
                    if (res.contentType === 'text/vnd.wap.wml') {
                        console.log(
                            "WMLRECORD" + "\t" +
                                url['url'] + "\t" +
                                url['headers']['X-CS'] + "\t" +
                                url['headers']['Accept'] + "\t" +
                                url['ua'] + "\t" +
                                url['safeDomains'] + "\t" +
                                url['safePartnerUrl'] + "\t" +
                                url['enabled']
                        );
                    }
                };

                return page.open(url['url'], function(status) {
                    if (status === "success") {
                        return window.setTimeout((function() {
                            return next(status, url);
                        }), 100);
                    } else {
                        if (url['ua'] !== 'Nokia3330/1.0 (03.05)') {
                            console.log("PAGE.OPEN FAILED : JSON.stringify(url) is " + JSON.stringify(url));
                        }
                        return next(status, url);
                    }
                });
            } else {
                return callbackFinal(getBucket, emptyBucket);
            }
        };
        return retrieve();
    };

    ZRMA.makeUrl = function(scheme, lang, subdomain, headers, ua, middle, bustcache, safeDomains, safePartnerUrl, enabled, comment, whitelistedLangs, subdomains) {

        var qRx = /\?/;
        var sep = qRx.test(middle) ? '&' : '?';

        var cachebuster = bustcache ?
            sep + 'cachebuster=' + xcs + '-' + (Math.floor(Math.random() * 999999) + 1) :
            '';

        return {
            'url' :
                scheme + '://' + lang + '.' + subdomain + '.' + 'wikipedia.org/wiki/' + middle + cachebuster,
            'headers' : headers,
            'ua' : ua,
            'safeDomains' : safeDomains,
            'safePartnerUrl' : safePartnerUrl,
            'enabled' : enabled,
            'subdomain' : subdomain,
            'comment' : comment,
            'whitelistedLangs' : whitelistedLangs,
            'subdomains' : subdomains,
            'lang' : lang
        };
    };

    ZRMA.initialArrayOfUrls = [

        {
            'url' : 'http://wikipedia.net/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://www.wikipedia.net/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://wikipedia.com/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://www.wikipedia.com/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://wikipedia.org/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://www.wikipedia.org/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://zero.wikipedia.org/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },

        {
            'url' : 'http://m.wikipedia.org/',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        },


        // this should be last in order to continue into the first callbackFinal below
        {
            'url' : 'https://meta.wikimedia.org/w/index.php?title=Special:PrefixIndex&prefix=&namespace=480',
            'headers' : {},
            'ua' : 'wikipedia-zero-auto-tests'
        }

    ];

    ZRMA.dt = ZRMA.mkdir();

    ZRMA.DoStuffWithUrls(ZRMA.initialArrayOfUrls, (function(status, url, page, fillBucketFunc, addToBucketFunc) {
        if (status !== "success") {
            return console.log("Unable to render '" + url['url'] + "'" + ' on ' + (new Date()).toISOString());
        } else {

            var startPage;

            switch (url['url']) {
                case 'http://wikipedia.net/' :
                    console.log('Loaded homepage from wikipedia.net - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000wikipedia.net.pdf';
                    break;
                case 'http://www.wikipedia.net/' :
                    console.log('Loaded homepage from www.wikipedia.net - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000www.wikipedia.net.pdf';
                    break;
                case 'http://wikipedia.com/' :
                    console.log('Loaded homepage from wikipedia.com - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000wikipedia.com.pdf';
                    break;
                case 'http://www.wikipedia.com/' :
                    console.log('Loaded homepage from www.wikipedia.com - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000www.wikipedia.com.pdf';
                    break;
                case 'http://wikipedia.org/' :
                    console.log('Loaded homepage from wikipedia.org - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000wikipedia.org.pdf';
                    break;
                case 'http://www.wikipedia.org/' :
                    console.log('Loaded homepage from www.wikipedia.org - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000www.wikipedia.org.pdf';
                    break;
                case 'http://zero.wikipedia.org/' :
                    console.log('Loaded homepage from zero.wikipedia.org - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000zero.wikipedia.org.pdf';
                    break;

                case 'http://m.wikipedia.org/' :
                    console.log('Loaded homepage from m.wikipedia.org - actual URL of ' + page.url + ' on ' + (new Date()).toISOString());
                    startPage = '000m.wikipedia.org.pdf';
                    break;

                // incidentally, above in ZRMA.initialArrayOfUrls, this should be last in order to continue into callbackFinal
                case 'https://meta.wikimedia.org/w/index.php?title=Special:PrefixIndex&prefix=&namespace=480':
                    console.log('LOADED MAIN CONFIGURATION PAGE AT ' + url['url'] + ' on ' + (new Date()).toISOString());
                    startPage = '000meta.pdf';
                    break;
            }

            page.render(ZRMA.dt + '/' + startPage);


            var results = page.evaluate(function(zeroConfigUrlPrefix) {

                var list = document.querySelectorAll('a'), hrefList = [];
                for (i = 0; i < list.length; i++) {
                    var href = list[i].href;
                    var match = zeroConfigUrlPrefix.exec(href);
                    if (match !== null) {
                        hrefList.push(match[1]);
                    }
                }
                return hrefList;
            }, ZRMA.getXcs());


            for (var i = 0; i < results.length; i++) {
                console.log('CONFIGURATION IDENTIFIED ' + results[i]);
            }

            fillBucketFunc(results);
            return;
        }
    }), function(getBucketFunc, emptyBucketFunc) {
        var b = getBucketFunc();
        emptyBucketFunc();

        ZRMA.DoStuffWithUrls(
            b.map( function(elem) {
                return {
                    'url' :
                        'https://meta.wikimedia.org/w/api.php?action=query&prop=revisions&rvlimit=1&rvprop=content&format=json&titles=Zero:' + elem,
                    'headers' : {},
                    'ua' : 'wikipedia-zero-auto-tests'
                };
            }),
            function(status, url, page, fillBucketFunc, addToBucketFunc) {
                if (status !== 'success') {
                    return console.log("Unable to render '" + url['url'] + "'" + ' on ' + (new Date()).toISOString());
                } else {
                    var fs = require('fs');
                    var fileName;
                    var rx = /^.+Zero:(.+)$/;
                    var xcs = rx.exec(url['url']);
                    fileName = xcs[1] + '-000blob';
                    page.render(ZRMA.dt + '/' + fileName + '.pdf');

                    blob = page.plainText;
                    // console.log('trying to JSON.parse on:' + blob);
                    json = JSON.parse(blob);
                    pagesParent = json['query']['pages'];
                    for (var prop in pagesParent) {
                        if (pagesParent.hasOwnProperty(prop)) {
                            configBlobString = pagesParent[prop]['revisions'][0]['*'];
                            // console.log('configBlobString:' + configBlobString);
                            configBlobJ = JSON.parse(configBlobString);
                            fs.write(ZRMA.dt + '/' + fileName + '.txt', JSON.stringify(configBlobJ), 'w');
                            configBlobJ['xcs'] = xcs[1];
                            addToBucketFunc(configBlobJ);
                        }
                    }

                    console.log('DOWNLOADED CONFIGURATION ' + url['url'] + ' to ' + fileName + ' on ' + (new Date()).toISOString());
                }

            },

            function(getBucketFunc, emptyBucketFunc) {
                var configContent = getBucketFunc();
                emptyBucketFunc();

                var speed = ZRMA.getSpeed();

                var safeMdotDomains = /^(m\.wikipedia\.org|.+?\.m\.wikipedia\.org|.+?\.zero\.wikipedia\.org|zero\.wikipedia\.org|bits\.wikimedia\.org|commons\.wikimedia\.org|upload\.wikimedia\.org|meta\.wikimedia\.org|geoiplookup\.wikimedia\.org|meta\.m\.wikimedia\.org|commons\.m\.wikimedia\.org)$/;

                var safeZerodotDomains = /^(.+?\.zero\.wikipedia\.org|zero\.wikipedia\.org|bits\.wikimedia\.org|commons\.wikimedia\.org|meta\.wikimedia\.org|geoiplookup\.wikimedia\.org|meta\.m\.wikimedia\.org|commons\.m\.wikimedia\.org)$/;

                var urls = [];

                for (var i = 0; i < configContent.length; i++) {
                    comment = configContent[i]['comment'];
                    xcs = configContent[i]['xcs'];
                    bannerUrl = configContent[i]['bannerUrl'];
                    bannerWarning = configContent[i]['bannerWarning'];
                    safePartnerUrl = bannerWarning === false && bannerUrl ? bannerUrl : null;
                    enabled = configContent[i]['enabled'] === undefined ? true : configContent[i]['enabled'];
                    whitelistedLangs = configContent[i]['whitelistedLangs'];
                    langs = whitelistedLangs.length === 0 ? configContent[i]['showLangs'] : whitelistedLangs.slice(0);
                    if (langs.indexOf('en') === -1) {
                        langs.push('en');
                    }
                    if (langs.indexOf('pl') === -1) {
                        langs.push('pl');
                    }
                    sites = configContent[i]['sites'];
                    subdomains = sites === undefined ?
                        ['m', 'zero'] :
                        sites.map( function(elem) {
                            if (elem === 'm.wikipedia') return 'm';
                            if (elem === 'zero.wikipedia') return 'zero';
                        });

                    console.log('COMMENT FOR ' + xcs + ' is ' + comment + ' on ' + (new Date()).toISOString());
                    // console.log(comment);
                    // console.log(xcs);
                    // console.log(langs.length);
                    // console.log(subdomains.length);

                    for (var s = 0; s < subdomains.length; s++) {
                        for (var l = 0; l < langs.length; l++) {
                            var enabledForPair = enabled && (whitelistedLangs.length === 0 || whitelistedLangs.indexOf(langs[l]) !== -1);
                            var safeDomains = subdomains[s] === 'zero' ? safeZerodotDomains : safeMdotDomains;

                            urls.push(
                                ZRMA.makeUrl(
                                    'http',
                                    langs[l],
                                    subdomains[s],
                                    {'X-CS': xcs},
                                    'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                    'Main_Page',
                                    true,
                                    safeDomains,
                                    safePartnerUrl,
                                    enabledForPair,
                                    comment,
                                    whitelistedLangs,
                                    subdomains
                                )
                            );

                            urls.push(
                                ZRMA.makeUrl(
                                    'http',
                                    langs[l],
                                    subdomains[s],
                                    {'X-CS': xcs},
                                    'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                    'Main_Page',
                                    false,
                                    safeDomains,
                                    safePartnerUrl,
                                    enabledForPair,
                                    comment,
                                    whitelistedLangs,
                                    subdomains
                                )
                            );

                            urls.push(
                                ZRMA.makeUrl(
                                    'http',
                                    langs[l],
                                    subdomains[s],
                                    {'X-CS': xcs},
                                    'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                    'Special:ZeroRatedMobileAccess?from=Google&to=https%3A%2F%2Fgoogle.com',
                                    true,
                                    safeDomains,
                                    safePartnerUrl,
                                    enabledForPair,
                                    comment,
                                    whitelistedLangs,
                                    subdomains
                                )
                            );


                            // WAP!

                            urls.push(
                                ZRMA.makeUrl(
                                    'http',
                                    langs[l],
                                    subdomains[s],
                                    {
                                        'X-CS' : xcs,
                                        'Accept' : 'text/vnd.wap.wml'
                                    },
                                    'Nokia3330/1.0 (03.05)',
                                    'San_Francisco',
                                    false,
                                    safeDomains,
                                    safePartnerUrl,
                                    enabledForPair,
                                    comment,
                                    whitelistedLangs,
                                    subdomains
                                )
                            );



                            if (speed === 'normal') {



                                // WAP!

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {
                                            'X-CS' : xcs,
                                            'Accept' : 'text/vnd.wap.wml'
                                        },
                                        'Nokia3330/1.0 (03.05)',
                                        'San_Francisco',
                                        true,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                // back to non-WAP


                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Special:ZeroRatedMobileAccess',
                                        true,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Special:ZeroRatedMobileAccess',
                                        false,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );


                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS': xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Cat',
                                        true,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS': xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Cat',
                                        false,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'File:Santa_Claus.jpg',
                                        true,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'File:Santa_Claus.jpg',
                                        false,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Google?renderZeroRatedRedirect=true&returnto=https%3A%2F%2Fwww.google.com',
                                        true,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Google?renderZeroRatedRedirect=true&returnto=https%3A%2F%2Fwww.google.com',
                                        false,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );



                                // there's no need to cachebust Special:Random, so only one of these here.

                                urls.push(
                                    ZRMA.makeUrl(
                                        'http',
                                        langs[l],
                                        subdomains[s],
                                        {'X-CS' : xcs},
                                        'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                                        'Special:Random',
                                        false,
                                        safeDomains,
                                        safePartnerUrl,
                                        enabledForPair,
                                        comment,
                                        whitelistedLangs,
                                        subdomains
                                    )
                                );
                            }
                        }
                    }

                    // TODO: Setup XFF here to mimic default redirect.
                    // These next two probably need safeDomains as well.
                    // But visual inspection of screenshots will do for now.
                    // Currently, without forging XFF, this may not redirect properly.
                    // Redirects will go to English every time without XFF forging.

                    urls.push(

                        {
                            'url' : 'http://zero.wikipedia.org/',
                            'headers' : { 'X-CS' : xcs },
                            'ua' : 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                            'enabled' : enabled && subdomains.indexOf('zero') !== -1
                        }
                    );

                    urls.push(

                        {
                            'url' : 'http://m.wikipedia.org/',
                            'headers' : { 'X-CS' : xcs },
                            'ua' : 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
                            'enabled' : enabled && subdomains.indexOf('m') !== -1
                        }
                    );
                }

                for (var u = 0; u < urls.length; u++) {
                    console.log('QUEUED URL ' + urls[u]['url']);
                }


                ZRMA.DoStuffWithUrls(
                    urls,
                    function(status, url, page, fillBucketFunc, addToBucketFunc) {
                        if (status !== 'success' && url['headers']['Accept'] !== 'text/vnd.wap.wml') {
                            return console.log("Unable to render '" + url['url'] + "'" + ' on ' + (new Date()).toISOString());
                        } else {
                            var fs = require('fs');
                            console.log('DOWNLOADING URL ' + url['url'] + ' on ' + (new Date()).toISOString());

                            var results = page.evaluate(function(u) {
                                var list = document.querySelectorAll('a'), hrefList = [];
                                for (i = 0; i < list.length; i++) {
                                    var href = list[i].href;
                                    var linkClass = list[i].getAttribute('class');
                                    var linkAllClasses = list[i].className.split(/\s+/);
                                    hrefList.push( {'sourceUrl' : u['url']  ,'href': href, 'xcs' : u['headers']['X-CS'], 'enabled' : u['enabled'], 'linkClass' : linkClass, 'linkAllClasses' : linkAllClasses} );
                                }
                                return hrefList;
                            }, url);

                            results.forEach(
                                function (elem, i, arr) {
                                    console.log('From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' found hyperlink ' + decodeURI(elem['href']) + ' on ' + (new Date()).toISOString());

                                    if (url['safePartnerUrl'] && elem['href'] === url['safePartnerUrl']) {
                                        console.log('OKAY URL MATCHED (whitelisted partner URL) WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' on ' + (new Date()).toISOString());
                                    } else if (elem['enabled'] === true) {
                                        hostExtractor = /^http:\/\/(.+?)\/.*$/;
                                        domain = hostExtractor.exec(elem['href']);

                                        if (url['safeDomains']) {

                                            var cacheBustedOrRandomRx = /cachebuster=\d+|Special:Random/;
                                            var cacheBustedOrRandomRxIndicator = cacheBustedOrRandomRx.test(elem['sourceUrl']) ?
                                                'CACHEBUSTED OR RANDOM REDIRECTED' :
                                                'NORMAL';

                                            if (domain === null) {
                                                emptyOrUrlIndicator = (elem['href'] === undefined || elem['href'] === '') ? 'with undefined or empty HREF' : decodeURI(elem['href']);
                                                console.log('UNSAFE URL UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + emptyOrUrlIndicator + ' using regex ' + hostExtractor.source + ' on ' + (new Date()).toISOString())
                                            } else {
                                                domainMatch = url['safeDomains'].exec(domain[1]);
                                                if (domainMatch === null) {
                                                    console.log('UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regex ' + url['safeDomains'].source + ' after ' + hostExtractor.source + ' identified domain ' + domain[1] + ' on ' + (new Date()).toISOString());
                                                } else {



                                                    var worthConsideration = false;
                                                    var parts = domain[1].split('.');
                                                    var pLength = parts.length;

                                                    if (pLength === 4 &&
                                                        url['subdomains'] !== undefined &&
                                                        url['whitelistedLangs'] !== undefined &&
                                                        parts[3] === 'org' &&
                                                        parts[2] === 'wikipedia' &&
                                                        url['subdomains'].indexOf(parts[1]) !== -1 &&
                                                        (url['whitelistedLangs'].length === 0 || url['whitelistedLangs'].indexOf(parts[0]) !== -1)
                                                        ) {

                                                        console.log('OKAY URL MATCHED (domain=' + domain[1] + ') WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw language-safe hyperlink ' + decodeURI(elem['href']) + ' using regexes ' + url['safeDomains'].source + ' and ' + hostExtractor.source + ' on ' + (new Date()).toISOString());

                                                        worthConsideration = true;


                                                    } else if (pLength === 4 &&
                                                        url['subdomains'] !== undefined &&
                                                        url['whitelistedLangs'] !== undefined &&
                                                        parts[3] === 'org' &&
                                                        parts[2] === 'wikipedia' &&
                                                        (url['subdomains'].indexOf(parts[1]) === -1 ||
                                                            (url['whitelistedLangs'].length > 0 &&
                                                            url['whitelistedLangs'].indexOf(parts[0]) === -1)
                                                        )

                                                    ) {
                                                        console.log('POTENTIALLY DANGEROUS LANGUAGE WIKIPEDIA LINK UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regex ' + url['safeDomains'].source + ' after ' + hostExtractor.source + ' identified domain ' + domain[1] + ' and langs list of ' + url['whitelistedLangs'].join(', ') + ' on ' + (new Date()).toISOString());
                                                    } else {
                                                        console.log('OKAY URL MATCHED (domain=' + domain[1] + ') WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regexes ' + url['safeDomains'].source + ' and ' + hostExtractor.source + ' on ' + (new Date()).toISOString());
                                                        worthConsideration = true;
                                                    }

                                                    if (worthConsideration) {
                                                        addToBucketFunc(
                                                            {
                                                                'urlObj' : url,
                                                                'actualUrl' : page.url,
                                                                'destinationUrl' : elem['href'],
                                                                'destinationDomain' : domain[1],
                                                                'linkClass' : elem['linkClass'],
                                                                'linkAllClasses' : elem['linkAllClasses']
                                                            }
                                                        );
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            );

                            patternMain = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Main_Page[?]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                            mM = patternMain.exec(url['url']);
                            if (mM !== null) {
                                fileName = mM[3] + '-' + mM[4] + '-' + mM[2] + '-' + mM[1] + '-Main_Page-Cachebusted';
                                page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());

                                page.clipRect = { top: 0, left: 0, width: 320, height: 150 };
                                page.render(ZRMA.dt + '/' + fileName + '-topclip.png');
                                console.log('SAVED FILE ' + fileName + '-topclip.png on ' + (new Date()).toISOString());
                                var wlLangs = url['whitelistedLangs'].length === 0 ? 'all langs' : url['whitelistedLangs'].join(", ");
                                var enabledVerbiage = url['enabled'] ? 'yes' : '<div style="color:red; font-weight:bold;">NO</div>';
                                var langSupportedVerbiage;
                                if (wlLangs === 'all langs') {
                                    langSupportedVerbiage = 'yes (all)';
                                } else {
                                    langSupportedVerbiage = ((url['whitelistedLangs']).indexOf(mM[1]) === -1) ?
                                        '<div style="color:red; font-weight:bold;">NO</div> (' + mM[1] + ')' :
                                        'yes (' + mM[1] + ')';
                                }
                                fs.write(
                                    ZRMA.dt + '/000.html',
                                    '<tr><td>' + url['comment'] + '</td>' +
                                    '<td>' + url['headers']['X-CS'] + '</td>' +
                                    '<td>' + url['url'] + '</td>' +
                                    '<td>' + url['subdomains'].join(", ") + '</td>' +
                                    '<td>' + wlLangs + '</td>' +
                                    '<td>' + enabledVerbiage + '</td>' +
                                    '<td>' + langSupportedVerbiage + '</td>' +
                                    '<td><img src="' + fileName + '-topclip.png" alt="' + fileName + '-topclip.png" /></td></tr>' ,
                                    'a'
                                );

                            }

                            patternMainNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Main_Page$/;
                            mMN = patternMainNobust.exec(url['url']);
                            if (mMN !== null) {
                                fileName = url['headers']['X-CS'] + '-' + mMN[2] + '-' + mMN[1] + '-Main_Page-Nobust';
                                page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());

                                page.clipRect = { top: 0, left: 0, width: 320, height: 150 };
                                page.render(ZRMA.dt + '/' + fileName + '-topclip.png');
                                console.log('SAVED FILE ' + fileName + '-topclip.png on ' + (new Date()).toISOString());

                                var wlLangs = url['whitelistedLangs'].length === 0 ? 'all langs' : url['whitelistedLangs'].join(", ");
                                var enabledVerbiage = url['enabled'] ? 'yes' : '<div style="color:red; font-weight:bold;">NO</div>';
                                var langSupportedVerbiage;
                                if (wlLangs === 'all langs') {
                                    langSupportedVerbiage = 'yes (all)';
                                } else {
                                    langSupportedVerbiage = ((url['whitelistedLangs']).indexOf(mMN[1]) === -1) ?
                                        '<div style="color:red; font-weight:bold;">NO</div> (' + mMN[1] + ')' :
                                        'yes (' + mMN[1] + ')';
                                }

                                fs.write(
                                    ZRMA.dt + '/000.html',
                                    '<tr><td>' + url['comment'] + '</td>' +
                                        '<td>' + url['headers']['X-CS'] + '</td>' +
                                        '<td>' + url['url'] + '</td>' +
                                        '<td>' + url['subdomains'].join(", ") + '</td>' +
                                        '<td>' + wlLangs + '</td>' +
                                        '<td>' + enabledVerbiage + '</td>' +
                                        '<td>' + langSupportedVerbiage + '</td>' +
                                        '<td><img src="' + fileName + '-topclip.png" alt="' + fileName + '-topclip.png" /></td></tr>' ,
                                    'a'
                                );
                            }


                            patternSanFranciscoNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+San_Francisco$/;
                            mWN = patternSanFranciscoNobust.exec(url['url']);
                            if (mWN !== null) {
                                fileName = url['headers']['X-CS'] + '-' + mWN[2] + '-' + mWN[1] + '-WAP-San_Francisco-Nobust';
                                page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                            }


                            if (speed === 'normal') {

                                patternCat = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Cat[?]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                                mC = patternCat.exec(url['url']);
                                if (mC !== null) {
                                    fileName = mC[3] + '-' + mC[4] + '-' + mC[2] + '-' + mC[1] + '-Cat-Cachebusted';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());

                                }

                                patternCatNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Cat$/;
                                mCN = patternCatNobust.exec(url['url']);
                                if (mCN !== null) {
                                    fileName = url['headers']['X-CS'] + '-' + mCN[2] + '-' + mCN[1] + '-Cat-Nobust';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());

                                }

                                patternSpecial = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Special:ZeroRatedMobileAccess[?]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                                mS = patternSpecial.exec(url['url']);
                                if (mS !== null) {
                                    fileName = mS[3] + '-' + mS[4] + '-' + mS[2] + '-' + mS[1] + '-SpecialZRMA-Cachebusted';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                                patternSpecialNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Special:ZeroRatedMobileAccess$/;
                                mSN = patternSpecialNobust.exec(url['url']);
                                if (mSN !== null) {
                                    fileName = url['headers']['X-CS'] + '-' + mSN[2] + '-' + mSN[1] + '-SpecialZRMA-Nobust';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                                patternFile = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+File:Santa_Claus\.jpg[?]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                                mF = patternFile.exec(url['url']);
                                if (mF !== null) {
                                    fileName = mF[3] + '-' + mF[4] + '-' + mF[2] + '-' + mF[1] + '-File_Santa_Claus_dot_jpg-Cachebusted';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                                patternFileNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+File:Santa_Claus\.jpg$/;
                                mFN = patternFileNobust.exec(url['url']);
                                if (mFN !== null) {
                                    fileName = url['headers']['X-CS'] + '-' + mFN[2] + '-' + mFN[1] + '-File_Santa_Claus_dot_jpg-Nobust';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                                patternExternalLink = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+www\.google\.com[&]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                                mE = patternExternalLink.exec(url['url']);
                                if (mE !== null) {
                                    fileName = mE[3] + '-' + mE[4] + '-' + mE[2] + '-' + mE[1] + '-External_Link-Cachebusted';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                                patternExternalLinkNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+www\.google\.com$/;
                                mEN = patternExternalLinkNobust.exec(url['url']);
                                if (mEN !== null) {
                                    fileName = url['headers']['X-CS'] + '-' + mEN[2] + '-' + mEN[1] + '-External_Link-Nobust';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }


                                // Special:Random isn't queued as a cachebusted URL, so only one file save here
                                patternSpecialRandomNobust = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+Special:Random$/;
                                mSRN = patternSpecialRandomNobust.exec(url['url']);
                                if (mSRN !== null) {
                                    fileName = url['headers']['X-CS'] + '-' + mSRN[2] + '-' + mSRN[1] + '-SpecialRandom-NobustOnly';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }


                                // WAP

                                patternSanFrancisco = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+San_Francisco[?]cachebuster=(\d+)-([0-9*]+)-\d+$/;
                                mW = patternSanFrancisco.exec(url['url']);
                                if (mW !== null) {
                                    fileName = mW[3] + '-' + mW[4] + '-' + mW[2] + '-' + mW[1] + '-WAP-San_Francisco-Cachebusted';
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                }

                            }



                            // TODO: Use X-Forwarded-For mechanism to get correct redirect. See above.
                            if (url['url'] === 'http://zero.wikipedia.org/' || url['url'] === 'http://m.wikipedia.org/') {

                                tlp = url['url'] === 'http://zero.wikipedia.org/' ? 'zero' : 'm';
                                purl = page.url;
                                console.log('Top level URL ' + url['url'] + ' redirected to ' + purl + ' on ' + (new Date()).toISOString());
                                patternDot = /^http:\/\/([a-zA-Z-]+)\.(m|zero).+$/;
                                mD = patternDot.exec(purl);
                                if (mD !== null) {
                                    fileName = url['headers']['X-CS'] + '-000' + tlp + 'dot-to-' + mD[1] + 'dot' + mD[2];
                                    page.render(ZRMA.dt + '/' + fileName + '.pdf');
                                    console.log('SAVED FILE ' + fileName + '.pdf on ' + (new Date()).toISOString());
                                    fs.write(ZRMA.dt + '/' + fileName + '.content.html', page.content, 'w');
                                    console.log('SAVED FILE ' + fileName + '.content.html on ' + (new Date()).toISOString());
                                } else {
                                    console.log('Something went wrong in redirect from ' + url['url'] + ' to ' + page.url + ' so screenshot broken' + ' on ' + (new Date()).toISOString());;
                                }

                            }


                            // console.log(page.plainText);
                            return;
                        }
                    },
                    function(getBucketFunc, emptyBucketFunc) {
                        var targetLinks = getBucketFunc();
                        var urlsToFollow = [];
                        emptyBucketFunc();
                        if (targetLinks.length === 0) {
                            console.log('...Exiting...' + ' on ' + (new Date()).toISOString());
                            console.log();
                            return phantom.exit();
                        }

                        numRandLinksToCheck = 1 + Math.ceil(targetLinks.length / 500);
                        // console.log('numRandLinksToCheck: ' + numRandLinksToCheck);

                        var randIndexes = [];
                        var randBillingIndexes = [];

                        var acceptBillingRx = /Special:ZeroRatedMobileAccess[&?]from=([^&]+)&to=(.+)$/;
                        for (var u = 0; u < targetLinks.length; u++) {

                            /*
                            WARNING!!!
                            If spidering out farther, take out the acceptbilling=yes regex in the following if conditional.
                            Don't want to hit too many external links.
                             */


                            if ((targetLinks[u]['linkClass'] === 'image' && targetLinks[u]['urlObj']['subdomain'] === 'zero') || acceptBillingRx.test(targetLinks[u]['destinationUrl']) || targetLinks[u]['linkAllClasses'].indexOf('external') !== -1) {
                                randBillingIndexes.push(u);
                            }
                        }

                        var numRandBillingLinksToCheck = 1 + Math.ceil(randBillingIndexes.length / 20);
                        if (randBillingIndexes.length < numRandBillingLinksToCheck) {
                            numRandBillingLinksToCheck = randBillingIndexes.length;
                        }

                        for (var nb = 0; nb < numRandBillingLinksToCheck; nb++) {
                            randomBillingIndex = Math.floor(Math.random() * (randBillingIndexes.length - 1));
                            if (randIndexes.indexOf(randomBillingIndex) === -1) {
                                randIndexes.push(randomBillingIndex);
                            } else {
                                nb--;
                            }
                        }

                        for (var n = 0; n < numRandLinksToCheck; n++) {
                            randomIndex = Math.floor(Math.random() * (targetLinks.length - 1));
                            // console.log('randomIndex: ' + randomIndex);
                            if (randIndexes.indexOf(randomIndex) === -1) {
                                randIndexes.push(randomIndex);
                            } else {
                                n--;
                            }
                        }


                        // TODO: in the future, need to use different rule based on interstitial path
                        for (var r = 0; r < randIndexes.length; r++) {
                            var urlToAugment = targetLinks[randIndexes[r]]['urlObj'];
                            urlToAugment['url'] = targetLinks[randIndexes[r]]['destinationUrl'];
                            urlToAugment['destinationDomain'] = targetLinks[randIndexes[r]]['destinationDomain'];
                            urlToAugment['linkClass'] = targetLinks[randIndexes[r]]['linkClass'];
                            urlToAugment['linkAllClasses'] = targetLinks[randIndexes[r]]['linkAllClasses'];
                            urlToAugment['source'] = targetLinks[randIndexes[r]]['actualUrl'];

                            var redirRx = /^http:\/\/([a-zA-Z-]+)\.(m|zero)\.wikipedia\.org\/(.*)(\?|\?.+&)acceptbilling=yes(.*)$/;

                            /*
                             If we spider farther out, we need to get more granular,
                             perhaps making another supplementary field for Wikimedia-oriented
                             subdomains that are safe, as well as a wildcard for other subdomains.
                             But if we're going one link deep, this should be sufficient
                             because the user has accepted billing, so...
                            */


                            if (redirRx.test(urlToAugment['url']) || acceptBillingRx.test(urlToAugment['source'])) {
                                urlToAugment['goAnywhere'] = true;
                            }


                            urlsToFollow.push(urlToAugment);
                        }

                        /*
                        for (var p = 0; p < urlsToFollow.length; p++) {
                            console.log(JSON.stringify(urlsToFollow[p]));
                            console.log(urlsToFollow[p]['safeDomains'].source);
                        }

                        console.log('SAFE LINK COUNT: ' + targetLinks.length);
                        */

                        for (var uF = 0; uF < urlsToFollow.length; uF++) {
                            console.log('QUEUED URL ' + urlsToFollow[uF]['url']);
                        }

                        ZRMA.DoStuffWithUrls(
                            urlsToFollow,
                            function(status, url, page, fillBucketFunc, addToBucketFunc) {

                                /*
                                We can spider out farther if goAnywhere isn't true.
                                */

                                if (status !== 'success') {
                                    return console.log("Unable to render '" + url['url'] + "'" + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                } else if (url['goAnywhere'] === true) {
                                    console.log('GOANYWHERE LINK VISITED: ' + url['url'] + ' actually at ' + page.url + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                } else {
                                    console.log('DOWNLOADING URL ' + url['url'] + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());

                                    var results = page.evaluate(function(u) {
                                        var list = document.querySelectorAll('a'), hrefList = [];
                                        for (i = 0; i < list.length; i++) {
                                            var href = list[i].href;
                                            var linkClass = list[i].getAttribute('class');
                                            hrefList.push( {'sourceUrl' : u['url']  ,'href': href, 'xcs' : u['headers']['X-CS'], 'enabled' : u['enabled'], 'linkClass' : linkClass} );
                                        }
                                        return hrefList;
                                    }, url);

                                    results.forEach(
                                        function (elem, i, arr) {
                                            console.log('From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' found hyperlink ' + decodeURI(elem['href']) + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());

                                            if (url['safePartnerUrl'] && elem['href'] === url['safePartnerUrl']) {
                                                console.log('OKAY URL MATCHED (whitelisted partner URL) WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                            } else {
                                                hostExtractor = /^http:\/\/(.+?)\/.*$/;
                                                domain = hostExtractor.exec(elem['href']);

                                                if (url['safeDomains']) {
                                                    worthConsideration = false;

                                                    if (domain === null) {
                                                        emptyOrUrlIndicator = (elem['href'] === undefined || elem['href'] === '') ? 'with undefined or empty HREF' : decodeURI(elem['href']);
                                                        console.log('UNSAFE URL UNABLE TO EXTRACT PROPER SCHEME AND DOMAIN WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + emptyOrUrlIndicator + ' using regex ' + hostExtractor.source + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString())
                                                    } else {
                                                        domainMatch = url['safeDomains'].exec(domain[1]);

                                                        if (redirRx.test(elem['href']) || acceptBillingRx.test(urlToAugment['source'])) {
                                                            worthConsideration = true;
                                                        } else if (domainMatch === null) {
                                                            console.log('UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regex ' + url['safeDomains'].source + ' after ' + hostExtractor.source + ' identified domain ' + domain[1] + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                                        } else {

                                                            var worthConsideration = false;
                                                            var parts = domain[1].split('.');
                                                            var pLength = parts.length;

                                                            if (pLength === 4 &&
                                                                url['subdomains'] !== undefined &&
                                                                url['whitelistedLangs'] !== undefined &&
                                                                parts[3] === 'org' &&
                                                                parts[2] === 'wikipedia' &&
                                                                url['subdomains'].indexOf(parts[1]) !== -1 &&
                                                                (url['whitelistedLangs'].length === 0 || url['whitelistedLangs'].indexOf(parts[0]) !== -1)
                                                                ) {

                                                                console.log('OKAY URL MATCHED (domain=' + domain[1] + ') WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw language-safe hyperlink ' + decodeURI(elem['href']) + ' using regexes ' + url['safeDomains'].source + ' and ' + hostExtractor.source + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                                                worthConsideration = true;


                                                            } else if (pLength === 4 &&
                                                                url['subdomains'] !== undefined &&
                                                                url['whitelistedLangs'] !== undefined &&
                                                                parts[3] === 'org' &&
                                                                parts[2] === 'wikipedia' &&
                                                                (url['subdomains'].indexOf(parts[1]) === -1 ||
                                                                    (url['whitelistedLangs'].length > 0 &&
                                                                        url['whitelistedLangs'].indexOf(parts[0]) === -1)
                                                                    )
                                                                ) {
                                                                console.log('POTENTIALLY DANGEROUS LANGUAGE WIKIPEDIA LINK UNSAFE URL SCHEME CORRECT BUT DOMAIN NOT MATCHED WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + cacheBustedOrRandomRxIndicator + ' ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regex ' + url['safeDomains'].source + ' after ' + hostExtractor.source + ' identified domain ' + domain[1] + ' and langs list of ' + url['whitelistedLangs'].join(', ') + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                                            } else {
                                                                console.log('OKAY URL MATCHED (domain=' + domain[1] + ') WITH X-CS ' + elem['xcs'] + ' (enabled=' + elem['enabled'] + ') : From ' + elem['sourceUrl'] + ' actually at ' + page.url + ' saw hyperlink ' + decodeURI(elem['href']) + ' using regexes ' + url['safeDomains'].source + ' and ' + hostExtractor.source + ' FROM SPIDERED PAGE on ' + (new Date()).toISOString());
                                                                worthConsideration = true;
                                                            }

                                                        }

                                                        if (worthConsideration) {
                                                            addToBucketFunc(
                                                                {
                                                                    'urlObj' : url,
                                                                    'actualUrl' : page.url,
                                                                    'destinationUrl' : elem['href'],
                                                                    'destinationDomain' : domain[1],
                                                                    'linkClass' : elem['linkClass']
                                                                }
                                                            );
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    );

                                    return;
                                }
                            },
                            function(getBucketFunc, emptyBucketFunc) {
                                var fs = require('fs');
                                fs.write(
                                    ZRMA.dt + '/000.html',
                                    '</table></div></body></html>' ,
                                    'a'
                                );
                                console.log('...Exiting...' + ' on ' + (new Date()).toISOString());
                                console.log();
                                return phantom.exit();
                            }
                        );
                    }
                );
            }
        );
    });
} catch (e) {
    console.log('Error encountered. ARGH.');
    console.log(e.name + ": " + e.message);
}
