/* global mw, jQuery */
( function ( M, $ ) {
    'use strict';
    $( function () {
        var httpsProxyingEnhanced = false, // whether Varnish handles HTTPS nicely
            hostname = document.location.hostname.toLowerCase(), // hostname of the current page
            isMdot = hostname.indexOf( '.m.' ) !== -1,
            isFromHttp = window.location.protocol.toLowerCase().indexOf( 'https' ) !== 0,
            getConfig = ( function () {
                // Singleton to hold zero configuration
                var instance;
                return function () {
                    if ( !instance ) {
                        instance = M.require( 'api' ).get( { action: 'zeroconfig', type: 'config' } );
                    }
                    return instance;
                };
            } )();

        $( '#page-secondary-actions .languageSelector' ).off();

        if ( isMdot ) {
            /* CSS isn't working for this element, and !important breaks stuff */
            $( '.icon-settings' ).show();
        }

        if ( !httpsProxyingEnhanced ) {
            if ( !isFromHttp ) {
                // we can't hide by default in CSS b/c of feature phones
                $( '#zero-rated-banner' ).hide();
            }
        } else {
            getConfig().done( function( conf ) {
                if ( !conf.enableHttps ) {
                    if ( !isFromHttp ) {
                        /* we can't hide by default in CSS b/c of feature phones */
                        $( '#zero-rated-banner' ).hide();
                    }
                } else {
                    if ( isMdot ) {
                        /*
                         FIXME: jdlrobson in change 83133 - "Maybe MobileFrontend can make this
                         easier by marking these links with a shared class so you can do something
                         like $( '.httpsLink' ).show();"
                         */
                        $( '.icon-uploads,.icon-watchlist,.icon-loginout,#ca-upload,.edit-page' ).show();
                    }
                    $( '#page-actions' ).show();
                }
            } );
        }

        $( 'a' ).on( 'click', function ( ev ) {
            var knownSites = [
                    // a hardcoded list of our sites - avoids api call if not listed
                    'm.wikipedia', 'zero.wikipedia'
                ],
                $target = $( this ), // onclick <a> element
                isSpecialCase = false, // true if interstitial should be shown despite not being a redirect
                search = $target.prop( 'search' ), // query part of the link target URL
                isImage = $target.hasClass( 'image' ), // is this a link to an image page
                showWarning, // true if a warning page should be shown
                toUrl, // target URL of the link
                toAsAnchor, // target URL as a DOM <a> object
                toHostParts, // hostname part of the URL
                toLang = false, // language of the destination site
                toSite = false, // non-language specific name of the site, e.g. "m.wikipedia"
                isLocal, // true if the target is a link to the current site
                isHttpsSwitch; // true if the link would switch from http to https

            if ( $target.attr( 'id' ) === 'mw-mf-last-modified' ) {
                isSpecialCase = true;
            }

            if ( isSpecialCase ) {
                // Special case link, need to show interstitial regardless
                toUrl = $target.prop( 'href' );
            } else if ( /[?&]title=[^:&]+:ZeroRatedMobileAccess(&|$)/.test( search ) ) {
                // Extract target URL from the redirect link if needed
                toUrl = /[&?]to=([^&]+)/.exec( search );
                if ( toUrl !== null ) {
                    toUrl = decodeURIComponent( toUrl[1] );
                }
            }
            if ( !toUrl ) {
                return; // Link was not recognized, don't handle
            }

            if ( M.settings.getUserSetting( 'zerodontask', true ) ) {
                // User chooses not to be notified, no need for any kinds of warnings
                showWarning = false;
            } else if ( isSpecialCase ) {
                // Special case link
                if ( !httpsProxyingEnhanced ) {
                    // https handling is disabled, no need to load config
                    if ( isFromHttp ) {
                        showWarning = true; // http -> https, warn
                    } else {
                        return; // this is https already, handle click as usual
                    }
                }
            } else {
                // Link to Special:Zero redirector
                // Use a DOM anchor trick to parse target URL
                toAsAnchor = $('<a>', { href: toUrl } )[0];
                if ( toAsAnchor.hostname ) {
                    toHostParts = toAsAnchor.hostname.toLowerCase();
                    isLocal = toHostParts === hostname;
                } else {
                    toHostParts = hostname;
                    isLocal = true;
                }
                toHostParts = toHostParts.split( '.', 3 );
                if ( toHostParts.length >= 3 ) {
                    toLang = toHostParts[0];
                    toSite = toHostParts[1] + '.' + toHostParts[2];
                }
                if ( !isLocal && ( !toSite || $.inArray( toSite, knownSites ) === -1 ) ) {
                    showWarning = true; // Navigating to an unknown site
                } else {
                    isHttpsSwitch = isFromHttp && toAsAnchor.protocol.toLowerCase().indexOf( 'https' ) === 0;
                    if ( isLocal && !isImage && !isHttpsSwitch ) {
                        showWarning = false; // This is a local link to a non-image page
                    }
                }
            }

            // At this point do custom <a> click navigation
            // it will be handled by navigate() called either directly or from a callback
            ev.preventDefault();

            function navigate( warn ) {
                if ( warn ) {
                    M.require( 'ZeroOverlay' );
                    window.location.hash = ( isImage ? '#zerofile/' : '#zerosite/' ) + toUrl;
                } else {
                    // Navigate directly to the target bypassing a server hit
                    window.location.href = toUrl;
                }
            }

            if ( showWarning === undefined ) {
                // We still don't know how to handle it, and need to make a server call
                getConfig().done( function( conf ) {
                    var showWarning;
                    if ( !conf.enabled ) {
                        showWarning = false;
                    } else if ( isSpecialCase ) {
                        // special case - only depends on https handling
                        showWarning = !conf.enableHttps; // this is implied: || !httpsProxyingEnhanced
                    } else if ( !isLocal ) {
                        // The logical construct here is "should we navigate without asking" and inverting that.
                        showWarning = !(
                            ( conf.whitelistedLangs &&
                                ( conf.whitelistedLangs.length === 0 ||
                                    $.inArray( toLang, conf.whitelistedLangs ) !== -1 ) ) &&
                            ( conf.sites &&
                                ( conf.sites.length === 0 ||
                                    $.inArray( toSite, conf.sites ) !== -1 ) ) &&
                            ( !isHttpsSwitch || ( httpsProxyingEnhanced && conf.enableHttps ) )
                        );
                    } else {
                        // link to an image
                        showWarning = !conf.showImages;
                    }
                    navigate( showWarning );
                } );
            } else {
                navigate( showWarning );
            }
        } );
    } );
}( mw.mobileFrontend, jQuery ) );
